import React, { useMemo, useState, useCallback, Fragment, useLayoutEffect, useEffect } from 'react';
import ArrowRightIcon from '@mui/icons-material/ArrowRight';
import CancelIcon from '@mui/icons-material/Cancel';
import CheckBoxIcon from '@mui/icons-material/CheckBox';
import ClearAllIcon from '@mui/icons-material/ClearAll';
import CloseIcon from '@mui/icons-material/Close';
import DensityMediumIcon from '@mui/icons-material/DensityMedium';
import DensitySmallIcon from '@mui/icons-material/DensitySmall';
import DoubleArrowDownIcon from '@mui/icons-material/KeyboardDoubleArrowDown';
import DynamicFeedIcon from '@mui/icons-material/DynamicFeed';
import EditIcon from '@mui/icons-material/Edit';
import ExpandLessIcon from '@mui/icons-material/ExpandLess';
import ExpandMoreIcon from '@mui/icons-material/ExpandMore';
import FilterAltIcon from '@mui/icons-material/FilterAlt';
import FilterAltOff from '@mui/icons-material/FilterAltOff';
import FilterListIcon from '@mui/icons-material/FilterList';
import FilterListOffIcon from '@mui/icons-material/FilterListOff';
import FullscreenExitIcon from '@mui/icons-material/FullscreenExit';
import FullscreenIcon from '@mui/icons-material/Fullscreen';
import MoreHorizIcon from '@mui/icons-material/MoreHoriz';
import MoreVertIcon from '@mui/icons-material/MoreVert';
import PushPinIcon from '@mui/icons-material/PushPin';
import RestartAltIcon from '@mui/icons-material/RestartAlt';
import SaveIcon from '@mui/icons-material/Save';
import SearchIcon from '@mui/icons-material/Search';
import SearchOffIcon from '@mui/icons-material/SearchOff';
import SortIcon from '@mui/icons-material/Sort';
import ViewColumnIcon from '@mui/icons-material/ViewColumn';
import VisibilityOffIcon from '@mui/icons-material/VisibilityOff';
import { createTable, useTableInstance, getCoreRowModel, getExpandedRowModel, getFacetedRowModel, getFilteredRowModel, getGroupedRowModel, getPaginationRowModel, getSortedRowModel, functionalUpdate } from '@tanstack/react-table';
import { Tooltip, IconButton, Menu, MenuItem, Box, FormControlLabel, Switch, Button, Divider, ListItemIcon, Checkbox, debounce, Collapse, TextField, InputAdornment, TablePagination, Chip, Alert, LinearProgress, useMediaQuery, Toolbar, lighten, alpha, TableCell, TableSortLabel, TableRow, TableHead, darken, Skeleton, TableBody, TableFooter, Table, TableContainer, Paper, Dialog, Grow } from '@mui/material';
import { rankItem, rankings } from '@tanstack/match-sorter-utils';

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var MRT_DefaultLocalization_EN = {
  actions: 'Actions',
  and: 'and',
  cancel: 'Cancel',
  changeFilterMode: 'Change filter mode',
  changeSearchMode: 'Change search mode',
  clearFilter: 'Clear filter',
  clearSearch: 'Clear search',
  clearSort: 'Clear sort',
  clickToCopy: 'Click to copy',
  columnActions: 'Column Actions',
  copiedToClipboard: 'Copied to clipboard',
  edit: 'Edit',
  expand: 'Expand',
  expandAll: 'Expand all',
  filterBetween: 'Between',
  filterByColumn: 'Filter by {column}',
  filterContains: 'Contains',
  filterEmpty: 'Empty',
  filterEndsWith: 'Ends With',
  filterEquals: 'Equals',
  filterFuzzy: 'Fuzzy',
  filterGreaterThan: 'Greater Than',
  filterLessThan: 'Less Than',
  filterMode: 'Filter Mode: {filterType}',
  filterNotEmpty: 'Not Empty',
  filterNotEquals: 'Not Equals',
  filterStartsWith: 'Starts With',
  filteringByColumn: 'Filtering by {column} - {filterType} {filterValue}',
  groupByColumn: 'Group by {column}',
  groupedBy: 'Grouped by ',
  hideAll: 'Hide all',
  hideColumn: 'Hide {column} column',
  max: 'Max',
  min: 'Min',
  pinToLeft: 'Pin to left',
  pinToRight: 'Pin to right',
  resetColumnSize: 'Reset column size',
  rowActions: 'Row Actions',
  rowNumber: '#',
  rowNumbers: 'Row Numbers',
  save: 'Save',
  search: 'Search',
  selectedCountOfRowCountRowsSelected: '{selectedCount} of {rowCount} row(s) selected',
  select: 'Select',
  showAll: 'Show all',
  showAllColumns: 'Show all columns',
  showHideColumns: 'Show/Hide columns',
  showHideFilters: 'Show/Hide filters',
  showHideSearch: 'Show/Hide search',
  sortByColumnAsc: 'Sort by {column} ascending',
  sortByColumnDesc: 'Sort by {column} descending',
  sortedByColumnAsc: 'Sorted by {column} ascending',
  sortedByColumnDesc: 'Sorted by {column} descending',
  thenBy: ', then by ',
  to: 'to',
  toggleDensePadding: 'Toggle dense padding',
  toggleFullScreen: 'Toggle full screen',
  toggleSelectAll: 'Toggle select all',
  toggleSelectRow: 'Toggle select row',
  ungroupByColumn: 'Ungroup by {column}',
  unpin: 'Unpin',
  unpinAll: 'Unpin all',
  unsorted: 'Unsorted'
};

var MRT_Default_Icons = {
  ArrowRightIcon: ArrowRightIcon,
  CancelIcon: CancelIcon,
  CheckBoxIcon: CheckBoxIcon,
  ClearAllIcon: ClearAllIcon,
  CloseIcon: CloseIcon,
  DensityMediumIcon: DensityMediumIcon,
  DensitySmallIcon: DensitySmallIcon,
  DoubleArrowDownIcon: DoubleArrowDownIcon,
  DynamicFeedIcon: DynamicFeedIcon,
  EditIcon: EditIcon,
  ExpandLessIcon: ExpandLessIcon,
  ExpandMoreIcon: ExpandMoreIcon,
  FilterAltIcon: FilterAltIcon,
  FilterAltOff: FilterAltOff,
  FilterListIcon: FilterListIcon,
  FilterListOffIcon: FilterListOffIcon,
  FullscreenExitIcon: FullscreenExitIcon,
  FullscreenIcon: FullscreenIcon,
  MoreHorizIcon: MoreHorizIcon,
  MoreVertIcon: MoreVertIcon,
  PushPinIcon: PushPinIcon,
  RestartAltIcon: RestartAltIcon,
  SaveIcon: SaveIcon,
  SearchIcon: SearchIcon,
  SearchOffIcon: SearchOffIcon,
  SortIcon: SortIcon,
  ViewColumnIcon: ViewColumnIcon,
  VisibilityOffIcon: VisibilityOffIcon
};

var MRT_ExpandAllButton = function MRT_ExpandAllButton(_ref) {
  var tableInstance = _ref.tableInstance;
  var getIsAllRowsExpanded = tableInstance.getIsAllRowsExpanded,
      getIsSomeRowsExpanded = tableInstance.getIsSomeRowsExpanded,
      getCanSomeRowsExpand = tableInstance.getCanSomeRowsExpand,
      getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      DoubleArrowDownIcon = _tableInstance$option.icons.DoubleArrowDownIcon,
      localization = _tableInstance$option.localization,
      renderDetailPanel = _tableInstance$option.renderDetailPanel,
      toggleAllRowsExpanded = tableInstance.toggleAllRowsExpanded;

  var _getState = getState(),
      isDensePadding = _getState.isDensePadding;

  return React.createElement(Tooltip, {
    arrow: true,
    enterDelay: 1000,
    enterNextDelay: 1000,
    title: localization.expandAll
  }, React.createElement(IconButton, {
    "aria-label": localization.expandAll,
    disabled: !getCanSomeRowsExpand() && !renderDetailPanel,
    onClick: function onClick() {
      return toggleAllRowsExpanded(!getIsAllRowsExpanded());
    },
    sx: {
      height: isDensePadding ? '1.75rem' : '2.25rem',
      width: isDensePadding ? '1.75rem' : '2.25rem'
    }
  }, React.createElement(DoubleArrowDownIcon, {
    style: {
      transform: "rotate(" + (getIsAllRowsExpanded() ? -180 : getIsSomeRowsExpanded() ? -90 : 0) + "deg)",
      transition: 'transform 0.2s'
    }
  })));
};

var MRT_ExpandButton = function MRT_ExpandButton(_ref) {
  var row = _ref.row,
      tableInstance = _ref.tableInstance;
  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      ExpandMoreIcon = _tableInstance$option.icons.ExpandMoreIcon,
      localization = _tableInstance$option.localization,
      onRowExpandChange = _tableInstance$option.onRowExpandChange,
      renderDetailPanel = _tableInstance$option.renderDetailPanel;

  var _getState = getState(),
      isDensePadding = _getState.isDensePadding;

  var handleToggleExpand = function handleToggleExpand(event) {
    row.toggleExpanded();
    onRowExpandChange == null ? void 0 : onRowExpandChange({
      event: event,
      row: row,
      tableInstance: tableInstance
    });
  };

  return React.createElement(Tooltip, {
    arrow: true,
    enterDelay: 1000,
    enterNextDelay: 1000,
    title: localization.expand
  }, React.createElement(IconButton, {
    "aria-label": localization.expand,
    disabled: !row.getCanExpand() && !renderDetailPanel,
    onClick: handleToggleExpand,
    sx: {
      height: isDensePadding ? '1.75rem' : '2.25rem',
      width: isDensePadding ? '1.75rem' : '2.25rem'
    }
  }, React.createElement(ExpandMoreIcon, {
    style: {
      transform: "rotate(" + (!row.getCanExpand() && !renderDetailPanel ? -90 : row.getIsExpanded() ? -180 : 0) + "deg)",
      transition: 'transform 0.2s'
    }
  })));
};

var MRT_FILTER_OPTION;

(function (MRT_FILTER_OPTION) {
  MRT_FILTER_OPTION["BETWEEN"] = "between";
  MRT_FILTER_OPTION["CONTAINS"] = "contains";
  MRT_FILTER_OPTION["EMPTY"] = "empty";
  MRT_FILTER_OPTION["ENDS_WITH"] = "endsWith";
  MRT_FILTER_OPTION["EQUALS"] = "equals";
  MRT_FILTER_OPTION["FUZZY"] = "fuzzy";
  MRT_FILTER_OPTION["GREATER_THAN"] = "greaterThan";
  MRT_FILTER_OPTION["LESS_THAN"] = "lessThan";
  MRT_FILTER_OPTION["NOT_EMPTY"] = "notEmpty";
  MRT_FILTER_OPTION["NOT_EQUALS"] = "notEquals";
  MRT_FILTER_OPTION["STARTS_WITH"] = "startsWith";
})(MRT_FILTER_OPTION || (MRT_FILTER_OPTION = {}));

var fuzzy = function fuzzy(row, columnId, value, addMeta) {
  var itemRank = rankItem(row.getValue(columnId), value, {
    threshold: rankings.MATCHES
  });
  addMeta(itemRank);
  return itemRank.passed;
};

fuzzy.autoRemove = function (val) {
  return !val;
};

var contains = function contains(row, id, filterValue) {
  return row.getValue(id).toString().toLowerCase().trim().includes(filterValue.toString().toLowerCase().trim());
};

contains.autoRemove = function (val) {
  return !val;
};

var startsWith = function startsWith(row, id, filterValue) {
  return row.getValue(id).toString().toLowerCase().trim().startsWith(filterValue.toString().toLowerCase().trim());
};

startsWith.autoRemove = function (val) {
  return !val;
};

var endsWith = function endsWith(row, id, filterValue) {
  return row.getValue(id).toString().toLowerCase().trim().endsWith(filterValue.toString().toLowerCase().trim());
};

endsWith.autoRemove = function (val) {
  return !val;
};

var equals = function equals(row, id, filterValue) {
  return row.getValue(id).toString().toLowerCase().trim() === filterValue.toString().toLowerCase().trim();
};

equals.autoRemove = function (val) {
  return !val;
};

var notEquals = function notEquals(row, id, filterValue) {
  return row.getValue(id).toString().toLowerCase().trim() !== filterValue.toString().toLowerCase().trim();
};

notEquals.autoRemove = function (val) {
  return !val;
};

var greaterThan = function greaterThan(row, id, filterValue) {
  return !isNaN(+filterValue) && !isNaN(+row.getValue(id)) ? +row.getValue(id) >= +filterValue : row.getValue(id).toString().toLowerCase().trim() > filterValue.toString().toLowerCase().trim();
};

greaterThan.autoRemove = function (val) {
  return !val;
};

var lessThan = function lessThan(row, id, filterValue) {
  return !isNaN(+filterValue) && !isNaN(+row.getValue(id)) ? +row.getValue(id) <= +filterValue : row.getValue(id).toString().toLowerCase().trim() < filterValue.toString().toLowerCase().trim();
};

lessThan.autoRemove = function (val) {
  return !val;
};

var between = function between(row, id, filterValues) {
  return (['', undefined].includes(filterValues[0]) || greaterThan(row, id, filterValues[0])) && (!isNaN(+filterValues[0]) && !isNaN(+filterValues[1]) && +filterValues[0] > +filterValues[1] || ['', undefined].includes(filterValues[1]) || lessThan(row, id, filterValues[1]));
};

between.autoRemove = function (val) {
  return !val;
};

var empty = function empty(row, id, _filterValue) {
  return !row.getValue(id).toString().trim();
};

empty.autoRemove = function (val) {
  return !val;
};

var notEmpty = function notEmpty(row, id, _filterValue) {
  return !!row.getValue(id).toString().trim();
};

notEmpty.autoRemove = function (val) {
  return !val;
};

var defaultFilterFNs = {
  between: between,
  contains: contains,
  empty: empty,
  endsWith: endsWith,
  equals: equals,
  fuzzy: fuzzy,
  greaterThan: greaterThan,
  lessThan: lessThan,
  notEmpty: notEmpty,
  notEquals: notEquals,
  startsWith: startsWith
};

var commonMenuItemStyles = {
  py: '6px',
  my: 0,
  alignItems: 'center'
};
var MRT_FilterOptionMenu = function MRT_FilterOptionMenu(_ref) {
  var anchorEl = _ref.anchorEl,
      header = _ref.header,
      onSelect = _ref.onSelect,
      setAnchorEl = _ref.setAnchorEl,
      tableInstance = _ref.tableInstance;
  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      enabledGlobalFilterOptions = _tableInstance$option.enabledGlobalFilterOptions,
      localization = _tableInstance$option.localization,
      setCurrentFilterFns = tableInstance.setCurrentFilterFns,
      setCurrentGlobalFilterFn = tableInstance.setCurrentGlobalFilterFn;

  var _getState = getState(),
      isDensePadding = _getState.isDensePadding,
      currentFilterFns = _getState.currentFilterFns,
      currentGlobalFilterFn = _getState.currentGlobalFilterFn;

  var filterOptions = useMemo(function () {
    return [{
      option: MRT_FILTER_OPTION.FUZZY,
      label: localization.filterFuzzy,
      divider: false,
      fn: fuzzy
    }, {
      option: MRT_FILTER_OPTION.CONTAINS,
      label: localization.filterContains,
      divider: false,
      fn: contains
    }, {
      option: MRT_FILTER_OPTION.STARTS_WITH,
      label: localization.filterStartsWith,
      divider: false,
      fn: startsWith
    }, {
      option: MRT_FILTER_OPTION.ENDS_WITH,
      label: localization.filterEndsWith,
      divider: true,
      fn: endsWith
    }, {
      option: MRT_FILTER_OPTION.EQUALS,
      label: localization.filterEquals,
      divider: false,
      fn: equals
    }, {
      option: MRT_FILTER_OPTION.NOT_EQUALS,
      label: localization.filterNotEquals,
      divider: true,
      fn: notEquals
    }, {
      option: MRT_FILTER_OPTION.BETWEEN,
      label: localization.filterBetween,
      divider: false,
      fn: between
    }, {
      option: MRT_FILTER_OPTION.GREATER_THAN,
      label: localization.filterGreaterThan,
      divider: false,
      fn: greaterThan
    }, {
      option: MRT_FILTER_OPTION.LESS_THAN,
      label: localization.filterLessThan,
      divider: true,
      fn: lessThan
    }, {
      option: MRT_FILTER_OPTION.EMPTY,
      label: localization.filterEmpty,
      divider: false,
      fn: empty
    }, {
      option: MRT_FILTER_OPTION.NOT_EMPTY,
      label: localization.filterNotEmpty,
      divider: false,
      fn: notEmpty
    }].filter(function (filterType) {
      return header ? !header.column.enabledColumnFilterOptions || header.column.enabledColumnFilterOptions.includes(filterType.option) : (!enabledGlobalFilterOptions || enabledGlobalFilterOptions.includes(filterType.option)) && [MRT_FILTER_OPTION.FUZZY, MRT_FILTER_OPTION.CONTAINS].includes(filterType.option);
    });
  }, []);

  var handleSelectFilterType = function handleSelectFilterType(value) {
    if (header) {
      setCurrentFilterFns(function (prev) {
        var _extends2;

        return _extends({}, prev, (_extends2 = {}, _extends2[header.id] = value, _extends2));
      });

      if ([MRT_FILTER_OPTION.EMPTY, MRT_FILTER_OPTION.NOT_EMPTY].includes(value)) {
        header.column.setFilterValue(' ');
      } else if (value === MRT_FILTER_OPTION.BETWEEN) {
        header.column.setFilterValue(['', '']);
      } else {
        header.column.setFilterValue('');
      }
    } else {
      setCurrentGlobalFilterFn(value);
    }

    setAnchorEl(null);
    onSelect == null ? void 0 : onSelect();
  };

  var filterOption = !!header ? currentFilterFns[header.id] : currentGlobalFilterFn;
  return React.createElement(Menu, {
    anchorEl: anchorEl,
    anchorOrigin: {
      vertical: 'center',
      horizontal: 'right'
    },
    onClose: function onClose() {
      return setAnchorEl(null);
    },
    open: !!anchorEl,
    MenuListProps: {
      dense: isDensePadding
    }
  }, filterOptions.map(function (_ref2, index) {
    var option = _ref2.option,
        label = _ref2.label,
        divider = _ref2.divider,
        fn = _ref2.fn;
    return React.createElement(MenuItem, {
      divider: divider,
      key: index,
      onClick: function onClick() {
        return handleSelectFilterType(option);
      },
      selected: option === filterOption || fn === filterOption,
      sx: commonMenuItemStyles,
      value: option
    }, label);
  }));
};

var MRT_ColumnPinningButtons = function MRT_ColumnPinningButtons(_ref) {
  var column = _ref.column,
      tableInstance = _ref.tableInstance;
  var _tableInstance$option = tableInstance.options,
      PushPinIcon = _tableInstance$option.icons.PushPinIcon,
      localization = _tableInstance$option.localization;

  var handlePinColumn = function handlePinColumn(pinDirection) {
    column.pin(pinDirection);
  };

  return React.createElement(Box, {
    sx: {
      mr: '8px'
    }
  }, column.getIsPinned() ? React.createElement(Tooltip, {
    arrow: true,
    title: localization.unpin
  }, React.createElement(IconButton, {
    onClick: function onClick() {
      return handlePinColumn(false);
    },
    size: "small"
  }, React.createElement(PushPinIcon, null))) : React.createElement(React.Fragment, null, React.createElement(Tooltip, {
    arrow: true,
    title: localization.pinToLeft
  }, React.createElement(IconButton, {
    onClick: function onClick() {
      return handlePinColumn('left');
    },
    size: "small"
  }, React.createElement(PushPinIcon, {
    style: {
      transform: 'rotate(90deg)'
    }
  }))), React.createElement(Tooltip, {
    arrow: true,
    title: localization.pinToRight
  }, React.createElement(IconButton, {
    onClick: function onClick() {
      return handlePinColumn('right');
    },
    size: "small"
  }, React.createElement(PushPinIcon, {
    style: {
      transform: 'rotate(-90deg)'
    }
  })))));
};

var MRT_ShowHideColumnsMenuItems = function MRT_ShowHideColumnsMenuItems(_ref) {
  var _column$columns2;

  var column = _ref.column,
      isSubMenu = _ref.isSubMenu,
      tableInstance = _ref.tableInstance;
  var getState = tableInstance.getState,
      onToggleColumnVisibility = tableInstance.options.onToggleColumnVisibility;

  var _getState = getState(),
      columnVisibility = _getState.columnVisibility;

  var switchChecked = column.columnDefType !== 'group' && column.getIsVisible() || column.columnDefType === 'group' && column.getLeafColumns().some(function (col) {
    return col.getIsVisible();
  });

  var handleToggleColumnHidden = function handleToggleColumnHidden(column) {
    if (column.columnDefType === 'group') {
      var _column$columns;

      column == null ? void 0 : (_column$columns = column.columns) == null ? void 0 : _column$columns.forEach == null ? void 0 : _column$columns.forEach(function (childColumn) {
        childColumn.toggleVisibility(!switchChecked);
      });
    } else {
      column.toggleVisibility();
    }

    onToggleColumnVisibility == null ? void 0 : onToggleColumnVisibility({
      column: column,
      columnVisibility: columnVisibility,
      tableInstance: tableInstance
    });
  };

  return React.createElement(React.Fragment, null, React.createElement(MenuItem, {
    sx: {
      alignItems: 'center',
      justifyContent: 'flex-start',
      my: 0,
      pl: (column.depth + 0.5) * 2 + "rem",
      py: '6px'
    }
  }, !isSubMenu && column.getCanPin() && React.createElement(MRT_ColumnPinningButtons, {
    column: column,
    tableInstance: tableInstance
  }), React.createElement(FormControlLabel, {
    componentsProps: {
      typography: {
        sx: {
          marginBottom: 0
        }
      }
    },
    checked: switchChecked,
    control: React.createElement(Switch, null),
    disabled: isSubMenu && switchChecked || !column.getCanHide(),
    label: column.columnDef.header,
    onChange: function onChange() {
      return handleToggleColumnHidden(column);
    }
  })), (_column$columns2 = column.columns) == null ? void 0 : _column$columns2.map(function (c, i) {
    return React.createElement(MRT_ShowHideColumnsMenuItems, {
      key: i + "-" + c.id,
      column: c,
      isSubMenu: isSubMenu,
      tableInstance: tableInstance
    });
  }));
};

var MRT_ShowHideColumnsMenu = function MRT_ShowHideColumnsMenu(_ref) {
  var anchorEl = _ref.anchorEl,
      isSubMenu = _ref.isSubMenu,
      setAnchorEl = _ref.setAnchorEl,
      tableInstance = _ref.tableInstance;
  var getAllColumns = tableInstance.getAllColumns,
      getIsAllColumnsVisible = tableInstance.getIsAllColumnsVisible,
      getIsSomeColumnsVisible = tableInstance.getIsSomeColumnsVisible,
      getIsSomeColumnsPinned = tableInstance.getIsSomeColumnsPinned,
      getState = tableInstance.getState,
      toggleAllColumnsVisible = tableInstance.toggleAllColumnsVisible,
      getAllLeafColumns = tableInstance.getAllLeafColumns,
      _tableInstance$option = tableInstance.options,
      localization = _tableInstance$option.localization,
      enablePinning = _tableInstance$option.enablePinning;

  var _getState = getState(),
      isDensePadding = _getState.isDensePadding;

  var hideAllColumns = function hideAllColumns() {
    getAllLeafColumns().filter(function (col) {
      return col.enableHiding !== false;
    }).forEach(function (col) {
      return col.toggleVisibility(false);
    });
  };

  var allDisplayColumns = useMemo(function () {
    return getAllColumns().filter(function (col) {
      return col.columnDefType === 'display';
    });
  }, [getAllColumns()]);
  var allDataColumns = useMemo(function () {
    var dataColumns = getAllColumns().filter(function (col) {
      return col.columnDefType !== 'display';
    });
    return getIsSomeColumnsPinned() ? [].concat(dataColumns.filter(function (c) {
      return c.getIsPinned() === 'left';
    }), [null], dataColumns.filter(function (c) {
      return c.getIsPinned() === false;
    }), [null], dataColumns.filter(function (c) {
      return c.getIsPinned() === 'right';
    })) : dataColumns;
  }, [getAllColumns(), getState().columnPinning, getIsSomeColumnsPinned()]);
  return React.createElement(Menu, {
    anchorEl: anchorEl,
    open: !!anchorEl,
    onClose: function onClose() {
      return setAnchorEl(null);
    },
    MenuListProps: {
      dense: isDensePadding
    }
  }, React.createElement(Box, {
    sx: {
      display: 'flex',
      justifyContent: isSubMenu ? 'center' : 'space-between',
      p: '0.5rem',
      pt: 0
    }
  }, !isSubMenu && React.createElement(Button, {
    disabled: !getIsSomeColumnsVisible(),
    onClick: hideAllColumns
  }, localization.hideAll), !isSubMenu && enablePinning && React.createElement(Button, {
    disabled: !getIsSomeColumnsPinned(),
    onClick: function onClick() {
      return tableInstance.resetColumnPinning(true);
    }
  }, localization.unpinAll), React.createElement(Button, {
    disabled: getIsAllColumnsVisible(),
    onClick: function onClick() {
      return toggleAllColumnsVisible(true);
    }
  }, localization.showAll)), React.createElement(Divider, null), allDisplayColumns.map(function (column, index) {
    return React.createElement(MRT_ShowHideColumnsMenuItems, {
      column: column,
      isSubMenu: isSubMenu,
      key: index + "-" + column.id,
      tableInstance: tableInstance
    });
  }), React.createElement(Divider, null), allDataColumns.map(function (column, index) {
    return column ? React.createElement(MRT_ShowHideColumnsMenuItems, {
      column: column,
      isSubMenu: isSubMenu,
      key: index + "-" + column.id,
      tableInstance: tableInstance
    }) : React.createElement(Divider, {
      key: index + "-divider"
    });
  }));
};

var commonMenuItemStyles$1 = {
  py: '6px',
  my: 0,
  justifyContent: 'space-between',
  alignItems: 'center'
};
var commonListItemStyles = {
  display: 'flex',
  alignItems: 'center'
};
var MRT_ColumnActionMenu = function MRT_ColumnActionMenu(_ref) {
  var _localization$sortByC, _localization$sortByC2, _localization$filterB, _localization, _localization$hideCol, _localization$showAll;

  var anchorEl = _ref.anchorEl,
      header = _ref.header,
      setAnchorEl = _ref.setAnchorEl,
      tableInstance = _ref.tableInstance;
  var getState = tableInstance.getState,
      toggleAllColumnsVisible = tableInstance.toggleAllColumnsVisible,
      setColumnOrder = tableInstance.setColumnOrder,
      _tableInstance$option = tableInstance.options,
      enableColumnFilters = _tableInstance$option.enableColumnFilters,
      enableColumnResizing = _tableInstance$option.enableColumnResizing,
      enableGrouping = _tableInstance$option.enableGrouping,
      enableHiding = _tableInstance$option.enableHiding,
      enablePinning = _tableInstance$option.enablePinning,
      enableSorting = _tableInstance$option.enableSorting,
      _tableInstance$option2 = _tableInstance$option.icons,
      ArrowRightIcon = _tableInstance$option2.ArrowRightIcon,
      ClearAllIcon = _tableInstance$option2.ClearAllIcon,
      ViewColumnIcon = _tableInstance$option2.ViewColumnIcon,
      DynamicFeedIcon = _tableInstance$option2.DynamicFeedIcon,
      FilterListIcon = _tableInstance$option2.FilterListIcon,
      FilterListOffIcon = _tableInstance$option2.FilterListOffIcon,
      PushPinIcon = _tableInstance$option2.PushPinIcon,
      SortIcon = _tableInstance$option2.SortIcon,
      RestartAltIcon = _tableInstance$option2.RestartAltIcon,
      VisibilityOffIcon = _tableInstance$option2.VisibilityOffIcon,
      idPrefix = _tableInstance$option.idPrefix,
      localization = _tableInstance$option.localization,
      setShowFilters = tableInstance.setShowFilters;
  var column = header.column;

  var _getState = getState(),
      columnSizing = _getState.columnSizing,
      columnVisibility = _getState.columnVisibility,
      isDensePadding = _getState.isDensePadding;

  var _useState = useState(null),
      filterMenuAnchorEl = _useState[0],
      setFilterMenuAnchorEl = _useState[1];

  var _useState2 = useState(null),
      showHideColumnsMenuAnchorEl = _useState2[0],
      setShowHideColumnsMenuAnchorEl = _useState2[1];

  var handleClearSort = function handleClearSort() {
    column.clearSorting();
    setAnchorEl(null);
  };

  var handleSortAsc = function handleSortAsc() {
    column.toggleSorting(false);
    setAnchorEl(null);
  };

  var handleSortDesc = function handleSortDesc() {
    column.toggleSorting(true);
    setAnchorEl(null);
  };

  var handleResetColumnSize = function handleResetColumnSize() {
    column.resetSize();
    setAnchorEl(null);
  };

  var handleHideColumn = function handleHideColumn() {
    column.toggleVisibility(false);
    setAnchorEl(null);
  };

  var handlePinColumn = function handlePinColumn(pinDirection) {
    column.pin(pinDirection);
    setAnchorEl(null);
  };

  var handleGroupByColumn = function handleGroupByColumn() {
    column.toggleGrouping();
    setColumnOrder(function (old) {
      return ['mrt-expand'].concat(old);
    });
    setAnchorEl(null);
  };

  var handleClearFilter = function handleClearFilter() {
    column.setFilterValue('');
    setAnchorEl(null);
  };

  var handleFilterByColumn = function handleFilterByColumn() {
    setShowFilters(true);
    setTimeout(function () {
      var _document$getElementB, _header$muiTableHeadC, _header$muiTableHeadC2;

      return (_document$getElementB = document.getElementById( // @ts-ignore
      (_header$muiTableHeadC = (_header$muiTableHeadC2 = header.muiTableHeadCellFilterTextFieldProps) == null ? void 0 : _header$muiTableHeadC2.id) != null ? _header$muiTableHeadC : "mrt-" + idPrefix + "-" + header.id + "-filter-text-field")) == null ? void 0 : _document$getElementB.focus();
    }, 200);
    setAnchorEl(null);
  };

  var handleShowAllColumns = function handleShowAllColumns() {
    toggleAllColumnsVisible(true);
    setAnchorEl(null);
  };

  var handleOpenFilterModeMenu = function handleOpenFilterModeMenu(event) {
    event.stopPropagation();
    setFilterMenuAnchorEl(event.currentTarget);
  };

  var handleOpenShowHideColumnsMenu = function handleOpenShowHideColumnsMenu(event) {
    event.stopPropagation();
    setShowHideColumnsMenuAnchorEl(event.currentTarget);
  };

  return React.createElement(Menu, {
    anchorEl: anchorEl,
    open: !!anchorEl,
    onClose: function onClose() {
      return setAnchorEl(null);
    },
    MenuListProps: {
      dense: isDensePadding
    }
  }, enableSorting && column.getCanSort() && [React.createElement(MenuItem, {
    disabled: !column.getIsSorted(),
    key: 0,
    onClick: handleClearSort,
    sx: commonMenuItemStyles$1
  }, React.createElement(Box, {
    sx: commonListItemStyles
  }, React.createElement(ListItemIcon, null, React.createElement(ClearAllIcon, null)), localization.clearSort)), React.createElement(MenuItem, {
    disabled: column.getIsSorted() === 'asc',
    key: 1,
    onClick: handleSortAsc,
    sx: commonMenuItemStyles$1
  }, React.createElement(Box, {
    sx: commonListItemStyles
  }, React.createElement(ListItemIcon, null, React.createElement(SortIcon, null)), (_localization$sortByC = localization.sortByColumnAsc) == null ? void 0 : _localization$sortByC.replace('{column}', String(column.columnDef.header)))), React.createElement(MenuItem, {
    divider: enableColumnFilters || enableGrouping || enableHiding,
    key: 2,
    disabled: column.getIsSorted() === 'desc',
    onClick: handleSortDesc,
    sx: commonMenuItemStyles$1
  }, React.createElement(Box, {
    sx: commonListItemStyles
  }, React.createElement(ListItemIcon, null, React.createElement(SortIcon, {
    style: {
      transform: 'rotate(180deg) scaleX(-1)'
    }
  })), (_localization$sortByC2 = localization.sortByColumnDesc) == null ? void 0 : _localization$sortByC2.replace('{column}', String(column.columnDef.header))))], enableColumnFilters && column.getCanFilter() && [React.createElement(MenuItem, {
    disabled: !column.getFilterValue(),
    key: 0,
    onClick: handleClearFilter,
    sx: commonMenuItemStyles$1
  }, React.createElement(Box, {
    sx: commonListItemStyles
  }, React.createElement(ListItemIcon, null, React.createElement(FilterListOffIcon, null)), localization.clearFilter)), React.createElement(MenuItem, {
    divider: enableGrouping || enableHiding,
    key: 1,
    onClick: handleFilterByColumn,
    sx: commonMenuItemStyles$1
  }, React.createElement(Box, {
    sx: commonListItemStyles
  }, React.createElement(ListItemIcon, null, React.createElement(FilterListIcon, null)), (_localization$filterB = localization.filterByColumn) == null ? void 0 : _localization$filterB.replace('{column}', String(column.columnDef.header))), !column.filterSelectOptions && React.createElement(IconButton, {
    onClick: handleOpenFilterModeMenu,
    onMouseEnter: handleOpenFilterModeMenu,
    size: "small",
    sx: {
      p: 0
    }
  }, React.createElement(ArrowRightIcon, null))), React.createElement(MRT_FilterOptionMenu, {
    anchorEl: filterMenuAnchorEl,
    header: header,
    key: 2,
    onSelect: handleFilterByColumn,
    setAnchorEl: setFilterMenuAnchorEl,
    tableInstance: tableInstance
  })], enableGrouping && column.getCanGroup() && [React.createElement(MenuItem, {
    divider: enablePinning,
    key: 0,
    onClick: handleGroupByColumn,
    sx: commonMenuItemStyles$1
  }, React.createElement(Box, {
    sx: commonListItemStyles
  }, React.createElement(ListItemIcon, null, React.createElement(DynamicFeedIcon, null)), (_localization = localization[column.getIsGrouped() ? 'ungroupByColumn' : 'groupByColumn']) == null ? void 0 : _localization.replace('{column}', String(column.columnDef.header))))], enablePinning && column.getCanPin() && [React.createElement(MenuItem, {
    disabled: column.getIsPinned() === 'left' || !column.getCanPin(),
    key: 0,
    onClick: function onClick() {
      return handlePinColumn('left');
    },
    sx: commonMenuItemStyles$1
  }, React.createElement(Box, {
    sx: commonListItemStyles
  }, React.createElement(ListItemIcon, null, React.createElement(PushPinIcon, {
    style: {
      transform: 'rotate(90deg)'
    }
  })), localization.pinToLeft)), React.createElement(MenuItem, {
    disabled: column.getIsPinned() === 'right' || !column.getCanPin(),
    key: 1,
    onClick: function onClick() {
      return handlePinColumn('right');
    },
    sx: commonMenuItemStyles$1
  }, React.createElement(Box, {
    sx: commonListItemStyles
  }, React.createElement(ListItemIcon, null, React.createElement(PushPinIcon, {
    style: {
      transform: 'rotate(-90deg)'
    }
  })), localization.pinToRight)), React.createElement(MenuItem, {
    disabled: !column.getIsPinned(),
    divider: enableHiding,
    key: 2,
    onClick: function onClick() {
      return handlePinColumn(false);
    },
    sx: commonMenuItemStyles$1
  }, React.createElement(Box, {
    sx: commonListItemStyles
  }, React.createElement(ListItemIcon, null, React.createElement(PushPinIcon, null)), localization.unpin))], enableColumnResizing && [React.createElement(MenuItem, {
    disabled: !column.getCanResize() || !columnSizing[column.id],
    key: 0,
    onClick: handleResetColumnSize,
    sx: commonMenuItemStyles$1
  }, React.createElement(Box, {
    sx: commonListItemStyles
  }, React.createElement(ListItemIcon, null, React.createElement(RestartAltIcon, null)), localization.resetColumnSize))], enableHiding && [React.createElement(MenuItem, {
    disabled: column.enableHiding === false,
    key: 0,
    onClick: handleHideColumn,
    sx: commonMenuItemStyles$1
  }, React.createElement(Box, {
    sx: commonListItemStyles
  }, React.createElement(ListItemIcon, null, React.createElement(VisibilityOffIcon, null)), (_localization$hideCol = localization.hideColumn) == null ? void 0 : _localization$hideCol.replace('{column}', String(column.columnDef.header)))), React.createElement(MenuItem, {
    disabled: !Object.values(columnVisibility).filter(function (visible) {
      return !visible;
    }).length,
    key: 1,
    onClick: handleShowAllColumns,
    sx: commonMenuItemStyles$1
  }, React.createElement(Box, {
    sx: commonListItemStyles
  }, React.createElement(ListItemIcon, null, React.createElement(ViewColumnIcon, null)), (_localization$showAll = localization.showAllColumns) == null ? void 0 : _localization$showAll.replace('{column}', String(column.columnDef.header))), React.createElement(IconButton, {
    onClick: handleOpenShowHideColumnsMenu,
    onMouseEnter: handleOpenShowHideColumnsMenu,
    size: "small",
    sx: {
      p: 0
    }
  }, React.createElement(ArrowRightIcon, null))), React.createElement(MRT_ShowHideColumnsMenu, {
    anchorEl: showHideColumnsMenuAnchorEl,
    isSubMenu: true,
    key: 2,
    setAnchorEl: setShowHideColumnsMenuAnchorEl,
    tableInstance: tableInstance
  })]);
};

var MRT_RowActionMenu = function MRT_RowActionMenu(_ref) {
  var anchorEl = _ref.anchorEl,
      handleEdit = _ref.handleEdit,
      row = _ref.row,
      setAnchorEl = _ref.setAnchorEl,
      tableInstance = _ref.tableInstance;
  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      EditIcon = _tableInstance$option.icons.EditIcon,
      enableEditing = _tableInstance$option.enableEditing,
      localization = _tableInstance$option.localization,
      renderRowActionMenuItems = _tableInstance$option.renderRowActionMenuItems;

  var _getState = getState(),
      isDensePadding = _getState.isDensePadding;

  return React.createElement(Menu, {
    anchorEl: anchorEl,
    open: !!anchorEl,
    onClose: function onClose() {
      return setAnchorEl(null);
    },
    MenuListProps: {
      dense: isDensePadding
    }
  }, enableEditing && React.createElement(MenuItem, {
    onClick: handleEdit,
    sx: commonMenuItemStyles$1
  }, React.createElement(Box, {
    sx: commonListItemStyles
  }, React.createElement(ListItemIcon, null, React.createElement(EditIcon, null)), localization.edit)), renderRowActionMenuItems == null ? void 0 : renderRowActionMenuItems({
    row: row,
    tableInstance: tableInstance,
    closeMenu: function closeMenu() {
      return setAnchorEl(null);
    }
  }));
};

var MRT_EditActionButtons = function MRT_EditActionButtons(_ref) {
  var row = _ref.row,
      tableInstance = _ref.tableInstance;
  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      _tableInstance$option2 = _tableInstance$option.icons,
      CancelIcon = _tableInstance$option2.CancelIcon,
      SaveIcon = _tableInstance$option2.SaveIcon,
      localization = _tableInstance$option.localization,
      onEditSubmit = _tableInstance$option.onEditSubmit,
      setCurrentEditingRow = tableInstance.setCurrentEditingRow;

  var _getState = getState(),
      currentEditingRow = _getState.currentEditingRow;

  var handleCancel = function handleCancel() {
    var _row$original;

    row._valuesCache = (_row$original = row.original) != null ? _row$original : {};
    setCurrentEditingRow(null);
  };

  var handleSave = function handleSave() {
    onEditSubmit == null ? void 0 : onEditSubmit({
      row: currentEditingRow != null ? currentEditingRow : row,
      tableInstance: tableInstance
    });
    setCurrentEditingRow(null);
  };

  return React.createElement(Box, {
    sx: {
      display: 'flex',
      gap: '0.75rem'
    }
  }, React.createElement(Tooltip, {
    arrow: true,
    title: localization.cancel
  }, React.createElement(IconButton, {
    "aria-label": localization.cancel,
    onClick: handleCancel
  }, React.createElement(CancelIcon, null))), React.createElement(Tooltip, {
    arrow: true,
    title: localization.save
  }, React.createElement(IconButton, {
    "aria-label": localization.save,
    color: "info",
    onClick: handleSave
  }, React.createElement(SaveIcon, null))));
};

var commonIconButtonStyles = {
  height: '2rem',
  ml: '10px',
  opacity: 0.5,
  transition: 'opacity 0.2s',
  width: '2rem',
  '&:hover': {
    opacity: 1
  }
};
var MRT_ToggleRowActionMenuButton = function MRT_ToggleRowActionMenuButton(_ref) {
  var row = _ref.row,
      tableInstance = _ref.tableInstance;
  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      enableEditing = _tableInstance$option.enableEditing,
      _tableInstance$option2 = _tableInstance$option.icons,
      EditIcon = _tableInstance$option2.EditIcon,
      MoreHorizIcon = _tableInstance$option2.MoreHorizIcon,
      localization = _tableInstance$option.localization,
      renderRowActionMenuItems = _tableInstance$option.renderRowActionMenuItems,
      renderRowActions = _tableInstance$option.renderRowActions,
      setCurrentEditingRow = tableInstance.setCurrentEditingRow;

  var _getState = getState(),
      currentEditingRow = _getState.currentEditingRow;

  var _useState = useState(null),
      anchorEl = _useState[0],
      setAnchorEl = _useState[1];

  var handleOpenRowActionMenu = function handleOpenRowActionMenu(event) {
    event.stopPropagation();
    event.preventDefault();
    setAnchorEl(event.currentTarget);
  };

  var handleStartEditMode = function handleStartEditMode() {
    setCurrentEditingRow(_extends({}, row));
    setAnchorEl(null);
  };

  return React.createElement(React.Fragment, null, renderRowActions ? React.createElement(React.Fragment, null, renderRowActions({
    row: row,
    tableInstance: tableInstance
  })) : row.id === (currentEditingRow == null ? void 0 : currentEditingRow.id) ? React.createElement(MRT_EditActionButtons, {
    row: row,
    tableInstance: tableInstance
  }) : !renderRowActionMenuItems && enableEditing ? React.createElement(Tooltip, {
    placement: "right",
    arrow: true,
    title: localization.edit
  }, React.createElement(IconButton, {
    sx: commonIconButtonStyles,
    onClick: handleStartEditMode
  }, React.createElement(EditIcon, null))) : renderRowActionMenuItems ? React.createElement(React.Fragment, null, React.createElement(Tooltip, {
    arrow: true,
    enterDelay: 1000,
    enterNextDelay: 1000,
    title: localization.rowActions
  }, React.createElement(IconButton, {
    "aria-label": localization.rowActions,
    onClick: handleOpenRowActionMenu,
    size: "small",
    sx: commonIconButtonStyles
  }, React.createElement(MoreHorizIcon, null))), React.createElement(MRT_RowActionMenu, {
    anchorEl: anchorEl,
    handleEdit: handleStartEditMode,
    row: row,
    setAnchorEl: setAnchorEl,
    tableInstance: tableInstance
  })) : null);
};

var MRT_SelectCheckbox = function MRT_SelectCheckbox(_ref) {
  var row = _ref.row,
      selectAll = _ref.selectAll,
      tableInstance = _ref.tableInstance;
  var getRowModel = tableInstance.getRowModel,
      getSelectedRowModel = tableInstance.getSelectedRowModel,
      getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      localization = _tableInstance$option.localization,
      muiSelectCheckboxProps = _tableInstance$option.muiSelectCheckboxProps,
      onSelectChange = _tableInstance$option.onSelectChange,
      onSelectAllChange = _tableInstance$option.onSelectAllChange;

  var _getState = getState(),
      isDensePadding = _getState.isDensePadding;

  var handleSelectChange = function handleSelectChange(event) {
    if (selectAll) {
      tableInstance.getToggleAllRowsSelectedHandler()(event);
      onSelectAllChange == null ? void 0 : onSelectAllChange({
        event: event,
        selectedRows: event.target.checked ? getRowModel().flatRows : [],
        tableInstance: tableInstance
      });
    } else if (row) {
      row == null ? void 0 : row.getToggleSelectedHandler()(event);
      onSelectChange == null ? void 0 : onSelectChange({
        event: event,
        row: row,
        selectedRows: event.target.checked ? [].concat(getSelectedRowModel().flatRows, [row]) : getSelectedRowModel().flatRows.filter(function (selectedRow) {
          return selectedRow.id !== row.id;
        }),
        tableInstance: tableInstance
      });
    }
  };

  var checkboxProps = muiSelectCheckboxProps instanceof Function ? muiSelectCheckboxProps({
    isSelectAll: !!selectAll,
    row: row,
    tableInstance: tableInstance
  }) : muiSelectCheckboxProps;
  return React.createElement(Tooltip, {
    arrow: true,
    enterDelay: 1000,
    enterNextDelay: 1000,
    title: selectAll ? localization.toggleSelectAll : localization.toggleSelectRow
  }, React.createElement(Checkbox, Object.assign({
    checked: selectAll ? tableInstance.getIsAllRowsSelected() : row == null ? void 0 : row.getIsSelected(),
    indeterminate: selectAll ? tableInstance.getIsSomeRowsSelected() : row == null ? void 0 : row.getIsSomeSelected(),
    inputProps: {
      'aria-label': selectAll ? localization.toggleSelectAll : localization.toggleSelectRow
    },
    onChange: handleSelectChange,
    size: isDensePadding ? 'small' : 'medium'
  }, checkboxProps, {
    sx: _extends({
      height: isDensePadding ? '1.75rem' : '2.25rem',
      width: isDensePadding ? '1.75rem' : '2.25rem'
    }, checkboxProps == null ? void 0 : checkboxProps.sx)
  })));
};

var _excluded = ["tableInstance"];
var MRT_FullScreenToggleButton = function MRT_FullScreenToggleButton(_ref) {
  var tableInstance = _ref.tableInstance,
      rest = _objectWithoutPropertiesLoose(_ref, _excluded);

  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      _tableInstance$option2 = _tableInstance$option.icons,
      FullscreenExitIcon = _tableInstance$option2.FullscreenExitIcon,
      FullscreenIcon = _tableInstance$option2.FullscreenIcon,
      localization = _tableInstance$option.localization,
      onToggleFullScreen = _tableInstance$option.onToggleFullScreen,
      setIsFullScreen = tableInstance.setIsFullScreen;

  var _getState = getState(),
      isFullScreen = _getState.isFullScreen;

  var handleToggleFullScreen = function handleToggleFullScreen(event) {
    onToggleFullScreen == null ? void 0 : onToggleFullScreen({
      event: event,
      isFullScreen: !isFullScreen,
      tableInstance: tableInstance
    });
    setIsFullScreen(!isFullScreen);
  };

  return React.createElement(Tooltip, {
    arrow: true,
    title: localization.toggleFullScreen
  }, React.createElement(IconButton, Object.assign({
    "aria-label": localization.showHideFilters,
    onClick: handleToggleFullScreen
  }, rest), isFullScreen ? React.createElement(FullscreenExitIcon, null) : React.createElement(FullscreenIcon, null)));
};

var _excluded$1 = ["tableInstance"];
var MRT_ShowHideColumnsButton = function MRT_ShowHideColumnsButton(_ref) {
  var tableInstance = _ref.tableInstance,
      rest = _objectWithoutPropertiesLoose(_ref, _excluded$1);

  var _tableInstance$option = tableInstance.options,
      ViewColumnIcon = _tableInstance$option.icons.ViewColumnIcon,
      localization = _tableInstance$option.localization;

  var _useState = useState(null),
      anchorEl = _useState[0],
      setAnchorEl = _useState[1];

  var handleClick = function handleClick(event) {
    setAnchorEl(event.currentTarget);
  };

  return React.createElement(React.Fragment, null, React.createElement(Tooltip, {
    arrow: true,
    title: localization.showHideColumns
  }, React.createElement(IconButton, Object.assign({
    "aria-label": localization.showHideColumns,
    onClick: handleClick
  }, rest), React.createElement(ViewColumnIcon, null))), React.createElement(MRT_ShowHideColumnsMenu, {
    anchorEl: anchorEl,
    setAnchorEl: setAnchorEl,
    tableInstance: tableInstance
  }));
};

var _excluded$2 = ["tableInstance"];
var MRT_ToggleDensePaddingButton = function MRT_ToggleDensePaddingButton(_ref) {
  var tableInstance = _ref.tableInstance,
      rest = _objectWithoutPropertiesLoose(_ref, _excluded$2);

  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      _tableInstance$option2 = _tableInstance$option.icons,
      DensityMediumIcon = _tableInstance$option2.DensityMediumIcon,
      DensitySmallIcon = _tableInstance$option2.DensitySmallIcon,
      localization = _tableInstance$option.localization,
      onToggleDensePadding = _tableInstance$option.onToggleDensePadding,
      setIsDensePadding = tableInstance.setIsDensePadding;

  var _getState = getState(),
      isDensePadding = _getState.isDensePadding;

  var handleToggleDensePadding = function handleToggleDensePadding(event) {
    onToggleDensePadding == null ? void 0 : onToggleDensePadding({
      event: event,
      isDensePadding: !isDensePadding,
      tableInstance: tableInstance
    });
    setIsDensePadding(!isDensePadding);
  };

  return React.createElement(Tooltip, {
    arrow: true,
    title: localization.toggleDensePadding
  }, React.createElement(IconButton, Object.assign({
    "aria-label": localization.toggleDensePadding,
    onClick: handleToggleDensePadding
  }, rest), isDensePadding ? React.createElement(DensitySmallIcon, null) : React.createElement(DensityMediumIcon, null)));
};

var _excluded$3 = ["tableInstance"];
var MRT_ToggleFiltersButton = function MRT_ToggleFiltersButton(_ref) {
  var tableInstance = _ref.tableInstance,
      rest = _objectWithoutPropertiesLoose(_ref, _excluded$3);

  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      _tableInstance$option2 = _tableInstance$option.icons,
      FilterListIcon = _tableInstance$option2.FilterListIcon,
      FilterListOffIcon = _tableInstance$option2.FilterListOffIcon,
      localization = _tableInstance$option.localization,
      onToggleShowFilters = _tableInstance$option.onToggleShowFilters,
      setShowFilters = tableInstance.setShowFilters;

  var _getState = getState(),
      showFilters = _getState.showFilters;

  var handleToggleShowFilters = function handleToggleShowFilters(event) {
    onToggleShowFilters == null ? void 0 : onToggleShowFilters({
      event: event,
      showFilters: !showFilters,
      tableInstance: tableInstance
    });
    setShowFilters(!showFilters);
  };

  return React.createElement(Tooltip, {
    arrow: true,
    title: localization.showHideFilters
  }, React.createElement(IconButton, Object.assign({
    "aria-label": localization.showHideFilters,
    onClick: handleToggleShowFilters
  }, rest), showFilters ? React.createElement(FilterListOffIcon, null) : React.createElement(FilterListIcon, null)));
};

var _excluded$4 = ["tableInstance"];
var MRT_ToggleGlobalFilterButton = function MRT_ToggleGlobalFilterButton(_ref) {
  var tableInstance = _ref.tableInstance,
      rest = _objectWithoutPropertiesLoose(_ref, _excluded$4);

  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      _tableInstance$option2 = _tableInstance$option.icons,
      SearchIcon = _tableInstance$option2.SearchIcon,
      SearchOffIcon = _tableInstance$option2.SearchOffIcon,
      idPrefix = _tableInstance$option.idPrefix,
      localization = _tableInstance$option.localization,
      muiSearchTextFieldProps = _tableInstance$option.muiSearchTextFieldProps,
      onToggleShowGlobalFilter = _tableInstance$option.onToggleShowGlobalFilter,
      setShowGlobalFilter = tableInstance.setShowGlobalFilter;

  var _getState = getState(),
      showGlobalFilter = _getState.showGlobalFilter;

  var textFieldProps = muiSearchTextFieldProps instanceof Function ? muiSearchTextFieldProps({
    tableInstance: tableInstance
  }) : muiSearchTextFieldProps;

  var handleToggleSearch = function handleToggleSearch(event) {
    onToggleShowGlobalFilter == null ? void 0 : onToggleShowGlobalFilter({
      event: event,
      showGlobalFilter: !showGlobalFilter,
      tableInstance: tableInstance
    });
    setShowGlobalFilter(!showGlobalFilter);
    setTimeout(function () {
      var _document$getElementB, _textFieldProps$id;

      return (_document$getElementB = document.getElementById((_textFieldProps$id = textFieldProps == null ? void 0 : textFieldProps.id) != null ? _textFieldProps$id : "mrt-" + idPrefix + "-search-text-field")) == null ? void 0 : _document$getElementB.focus();
    }, 200);
  };

  return React.createElement(Tooltip, {
    arrow: true,
    title: localization.showHideSearch
  }, React.createElement(IconButton, Object.assign({
    onClick: handleToggleSearch
  }, rest), showGlobalFilter ? React.createElement(SearchOffIcon, null) : React.createElement(SearchIcon, null)));
};

var MRT_SearchTextField = function MRT_SearchTextField(_ref) {
  var tableInstance = _ref.tableInstance;
  var getState = tableInstance.getState,
      setGlobalFilter = tableInstance.setGlobalFilter,
      _tableInstance$option = tableInstance.options,
      _tableInstance$option2 = _tableInstance$option.icons,
      SearchIcon = _tableInstance$option2.SearchIcon,
      CloseIcon = _tableInstance$option2.CloseIcon,
      idPrefix = _tableInstance$option.idPrefix,
      localization = _tableInstance$option.localization,
      muiSearchTextFieldProps = _tableInstance$option.muiSearchTextFieldProps,
      onGlobalSearchFilterChange = _tableInstance$option.onGlobalSearchFilterChange;

  var _getState = getState(),
      globalFilter = _getState.globalFilter,
      showGlobalFilter = _getState.showGlobalFilter;

  var _useState = useState(null),
      anchorEl = _useState[0],
      setAnchorEl = _useState[1];

  var _useState2 = useState(globalFilter != null ? globalFilter : ''),
      searchValue = _useState2[0],
      setSearchValue = _useState2[1];

  var handleChange = useCallback(debounce(function (event) {
    var _event$target$value;

    setGlobalFilter((_event$target$value = event.target.value) != null ? _event$target$value : undefined);
    onGlobalSearchFilterChange == null ? void 0 : onGlobalSearchFilterChange({
      event: event,
      tableInstance: tableInstance
    });
  }, 200), []);

  var handleGlobalFilterMenuOpen = function handleGlobalFilterMenuOpen(event) {
    setAnchorEl(event.currentTarget);
  };

  var handleClear = function handleClear() {
    setSearchValue('');
    setGlobalFilter(undefined);
  };

  var textFieldProps = muiSearchTextFieldProps instanceof Function ? muiSearchTextFieldProps({
    tableInstance: tableInstance
  }) : muiSearchTextFieldProps;
  return React.createElement(Collapse, {
    "in": showGlobalFilter,
    orientation: "horizontal"
  }, React.createElement(TextField, Object.assign({
    id: "mrt-" + idPrefix + "-search-text-field",
    placeholder: localization.search,
    onChange: function onChange(event) {
      setSearchValue(event.target.value);
      handleChange(event);
    },
    value: searchValue != null ? searchValue : '',
    variant: "standard",
    InputProps: {
      startAdornment: React.createElement(InputAdornment, {
        position: "start"
      }, React.createElement(Tooltip, {
        arrow: true,
        title: localization.changeSearchMode
      }, React.createElement("span", null, React.createElement(IconButton, {
        "aria-label": localization.changeSearchMode,
        onClick: handleGlobalFilterMenuOpen,
        size: "small",
        sx: {
          height: '1.75rem',
          width: '1.75rem'
        }
      }, React.createElement(SearchIcon, null))))),
      endAdornment: React.createElement(InputAdornment, {
        position: "end"
      }, React.createElement(IconButton, {
        "aria-label": localization.clearSearch,
        disabled: (searchValue == null ? void 0 : searchValue.length) === 0,
        onClick: handleClear,
        size: "small",
        title: localization.clearSearch
      }, React.createElement(CloseIcon, null)))
    }
  }, textFieldProps, {
    sx: _extends({
      justifySelf: 'end'
    }, textFieldProps == null ? void 0 : textFieldProps.sx)
  })), React.createElement(MRT_FilterOptionMenu, {
    anchorEl: anchorEl,
    setAnchorEl: setAnchorEl,
    tableInstance: tableInstance
  }));
};

var MRT_ToolbarInternalButtons = function MRT_ToolbarInternalButtons(_ref) {
  var _renderToolbarInterna;

  var tableInstance = _ref.tableInstance;
  var _tableInstance$option = tableInstance.options,
      enableColumnFilters = _tableInstance$option.enableColumnFilters,
      enableDensePaddingToggle = _tableInstance$option.enableDensePaddingToggle,
      enableFilters = _tableInstance$option.enableFilters,
      enableFullScreenToggle = _tableInstance$option.enableFullScreenToggle,
      enableGlobalFilter = _tableInstance$option.enableGlobalFilter,
      enableHiding = _tableInstance$option.enableHiding,
      renderToolbarInternalActions = _tableInstance$option.renderToolbarInternalActions;
  return React.createElement(Box, {
    sx: {
      display: 'flex',
      alignItems: 'center',
      zIndex: 3
    }
  }, (_renderToolbarInterna = renderToolbarInternalActions == null ? void 0 : renderToolbarInternalActions({
    MRT_FullScreenToggleButton: MRT_FullScreenToggleButton,
    MRT_ShowHideColumnsButton: MRT_ShowHideColumnsButton,
    MRT_ToggleDensePaddingButton: MRT_ToggleDensePaddingButton,
    MRT_ToggleFiltersButton: MRT_ToggleFiltersButton,
    MRT_ToggleGlobalFilterButton: MRT_ToggleGlobalFilterButton,
    tableInstance: tableInstance
  })) != null ? _renderToolbarInterna : React.createElement(React.Fragment, null, enableGlobalFilter && React.createElement(MRT_SearchTextField, {
    tableInstance: tableInstance
  }), enableFilters && enableGlobalFilter && React.createElement(MRT_ToggleGlobalFilterButton, {
    tableInstance: tableInstance
  }), enableFilters && enableColumnFilters && React.createElement(MRT_ToggleFiltersButton, {
    tableInstance: tableInstance
  }), enableHiding && React.createElement(MRT_ShowHideColumnsButton, {
    tableInstance: tableInstance
  }), enableDensePaddingToggle && React.createElement(MRT_ToggleDensePaddingButton, {
    tableInstance: tableInstance
  }), enableFullScreenToggle && React.createElement(MRT_FullScreenToggleButton, {
    tableInstance: tableInstance
  })));
};

var MRT_TablePagination = function MRT_TablePagination(_ref) {
  var tableInstance = _ref.tableInstance;
  var getPrePaginationRowModel = tableInstance.getPrePaginationRowModel,
      getState = tableInstance.getState,
      setPageIndex = tableInstance.setPageIndex,
      setPageSize = tableInstance.setPageSize,
      muiTablePaginationProps = tableInstance.options.muiTablePaginationProps;

  var _getState = getState(),
      _getState$pagination = _getState.pagination,
      _getState$pagination$ = _getState$pagination.pageSize,
      pageSize = _getState$pagination$ === void 0 ? 10 : _getState$pagination$,
      _getState$pagination$2 = _getState$pagination.pageIndex,
      pageIndex = _getState$pagination$2 === void 0 ? 0 : _getState$pagination$2;

  var tablePaginationProps = muiTablePaginationProps instanceof Function ? muiTablePaginationProps({
    tableInstance: tableInstance
  }) : muiTablePaginationProps;

  var handleChangeRowsPerPage = function handleChangeRowsPerPage(event) {
    setPageSize(+event.target.value);
  };

  var showFirstLastPageButtons = getPrePaginationRowModel().rows.length / pageSize > 2;
  return React.createElement(TablePagination, Object.assign({
    SelectProps: {
      sx: {
        m: '0 1rem 0 1ch'
      },
      MenuProps: {
        MenuListProps: {
          disablePadding: true
        }
      }
    },
    component: "div",
    count: getPrePaginationRowModel().rows.length,
    onPageChange: function onPageChange(_, newPage) {
      return setPageIndex(newPage);
    },
    onRowsPerPageChange: handleChangeRowsPerPage,
    page: pageIndex,
    rowsPerPage: pageSize,
    rowsPerPageOptions: [5, 10, 15, 20, 25, 30, 50, 100],
    showFirstButton: showFirstLastPageButtons,
    showLastButton: showFirstLastPageButtons
  }, tablePaginationProps, {
    sx: _extends({
      m: '0 0.5rem',
      position: 'relative',
      zIndex: 2
    }, tablePaginationProps == null ? void 0 : tablePaginationProps.sx)
  }));
};

var MRT_ToolbarAlertBanner = function MRT_ToolbarAlertBanner(_ref) {
  var _localization$selecte, _localization$selecte2;

  var stackAlertBanner = _ref.stackAlertBanner,
      tableInstance = _ref.tableInstance;
  var getPrePaginationRowModel = tableInstance.getPrePaginationRowModel,
      getSelectedRowModel = tableInstance.getSelectedRowModel,
      getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      localization = _tableInstance$option.localization,
      muiTableToolbarAlertBannerProps = _tableInstance$option.muiTableToolbarAlertBannerProps;

  var _getState = getState(),
      grouping = _getState.grouping;

  var alertProps = muiTableToolbarAlertBannerProps instanceof Function ? muiTableToolbarAlertBannerProps({
    tableInstance: tableInstance
  }) : muiTableToolbarAlertBannerProps;
  var selectMessage = getSelectedRowModel().rows.length > 0 ? (_localization$selecte = localization.selectedCountOfRowCountRowsSelected) == null ? void 0 : (_localization$selecte2 = _localization$selecte.replace('{selectedCount}', getSelectedRowModel().rows.length.toString())) == null ? void 0 : _localization$selecte2.replace('{rowCount}', getPrePaginationRowModel().rows.length.toString()) : null;
  var groupedByMessage = grouping.length > 0 ? React.createElement("span", null, localization.groupedBy, ' ', grouping.map(function (columnId, index) {
    return React.createElement(Fragment, {
      key: index + "-" + columnId
    }, index > 0 ? localization.thenBy : '', React.createElement(Chip, {
      color: "secondary",
      label: tableInstance.getColumn(columnId).columnDef.header,
      onDelete: function onDelete() {
        return tableInstance.getColumn(columnId).toggleGrouping();
      }
    }));
  })) : null;
  return React.createElement(Collapse, {
    "in": !!selectMessage || !!groupedByMessage,
    timeout: stackAlertBanner ? 200 : 0
  }, React.createElement(Alert, Object.assign({
    color: "info",
    icon: false,
    sx: _extends({
      borderRadius: 0,
      fontSize: '1rem',
      left: 0,
      p: 0,
      position: 'relative',
      right: 0,
      top: 0,
      width: '100%',
      zIndex: 2
    }, alertProps == null ? void 0 : alertProps.sx)
  }, alertProps), React.createElement(Box, {
    sx: {
      p: '0.5rem 1rem'
    }
  }, selectMessage, selectMessage && groupedByMessage && React.createElement("br", null), groupedByMessage)));
};

var MRT_LinearProgressBar = function MRT_LinearProgressBar(_ref) {
  var tableInstance = _ref.tableInstance;
  var muiLinearProgressProps = tableInstance.options.muiLinearProgressProps,
      getState = tableInstance.getState;

  var _getState = getState(),
      isLoading = _getState.isLoading,
      showProgressBars = _getState.showProgressBars;

  var linearProgressProps = muiLinearProgressProps instanceof Function ? muiLinearProgressProps({
    tableInstance: tableInstance
  }) : muiLinearProgressProps;
  return React.createElement(Collapse, {
    "in": isLoading || showProgressBars,
    mountOnEnter: true,
    unmountOnExit: true
  }, React.createElement(LinearProgress, Object.assign({
    "aria-label": "Loading",
    "aria-busy": "true"
  }, linearProgressProps)));
};

var commonToolbarStyles = function commonToolbarStyles(_ref) {
  var theme = _ref.theme;
  return {
    backgroundColor: lighten(theme.palette.background["default"], 0.04),
    backgroundImage: 'none',
    display: 'grid',
    minHeight: '3.5rem',
    overflow: 'hidden',
    p: '0 !important',
    transition: 'all 0.2s ease-in-out',
    width: '100%',
    zIndex: 1
  };
};
var MRT_ToolbarTop = function MRT_ToolbarTop(_ref2) {
  var _renderToolbarCustomA;

  var tableInstance = _ref2.tableInstance;
  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      enablePagination = _tableInstance$option.enablePagination,
      enableToolbarInternalActions = _tableInstance$option.enableToolbarInternalActions,
      idPrefix = _tableInstance$option.idPrefix,
      muiTableToolbarTopProps = _tableInstance$option.muiTableToolbarTopProps,
      positionPagination = _tableInstance$option.positionPagination,
      positionToolbarActions = _tableInstance$option.positionToolbarActions,
      positionToolbarAlertBanner = _tableInstance$option.positionToolbarAlertBanner,
      renderToolbarCustomActions = _tableInstance$option.renderToolbarCustomActions;

  var _getState = getState(),
      isFullScreen = _getState.isFullScreen;

  var isMobile = useMediaQuery('(max-width:720px)');
  var toolbarProps = muiTableToolbarTopProps instanceof Function ? muiTableToolbarTopProps({
    tableInstance: tableInstance
  }) : muiTableToolbarTopProps;
  var stackAlertBanner = isMobile || positionToolbarAlertBanner === 'top' && !!renderToolbarCustomActions;
  return React.createElement(Toolbar, Object.assign({
    id: "mrt-" + idPrefix + "-toolbar-top",
    variant: "dense"
  }, toolbarProps, {
    sx: function sx(theme) {
      return _extends({
        position: isFullScreen ? 'sticky' : undefined,
        top: isFullScreen ? '0' : undefined
      }, commonToolbarStyles({
        theme: theme
      }), toolbarProps == null ? void 0 : toolbarProps.sx);
    }
  }), positionToolbarAlertBanner === 'top' && React.createElement(MRT_ToolbarAlertBanner, {
    stackAlertBanner: stackAlertBanner,
    tableInstance: tableInstance
  }), React.createElement(Box, {
    sx: {
      display: 'flex',
      justifyContent: 'space-between',
      p: '0.5rem',
      position: stackAlertBanner ? 'relative' : 'absolute',
      right: 0,
      top: 0,
      width: 'calc(100% - 1rem)'
    }
  }, (_renderToolbarCustomA = renderToolbarCustomActions == null ? void 0 : renderToolbarCustomActions({
    tableInstance: tableInstance
  })) != null ? _renderToolbarCustomA : React.createElement("span", null), enableToolbarInternalActions && positionToolbarActions === 'top' && React.createElement(MRT_ToolbarInternalButtons, {
    tableInstance: tableInstance
  })), React.createElement("div", null, enablePagination && ['top', 'both'].includes(positionPagination != null ? positionPagination : '') && React.createElement(MRT_TablePagination, {
    tableInstance: tableInstance
  })), React.createElement(MRT_LinearProgressBar, {
    tableInstance: tableInstance
  }));
};

var MRT_ToolbarBottom = function MRT_ToolbarBottom(_ref) {
  var tableInstance = _ref.tableInstance;
  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      enableToolbarInternalActions = _tableInstance$option.enableToolbarInternalActions,
      idPrefix = _tableInstance$option.idPrefix,
      enablePagination = _tableInstance$option.enablePagination,
      muiTableToolbarBottomProps = _tableInstance$option.muiTableToolbarBottomProps,
      positionPagination = _tableInstance$option.positionPagination,
      positionToolbarActions = _tableInstance$option.positionToolbarActions,
      positionToolbarAlertBanner = _tableInstance$option.positionToolbarAlertBanner,
      renderToolbarCustomActions = _tableInstance$option.renderToolbarCustomActions;

  var _getState = getState(),
      isFullScreen = _getState.isFullScreen;

  var isMobile = useMediaQuery('(max-width:720px)');
  var toolbarProps = muiTableToolbarBottomProps instanceof Function ? muiTableToolbarBottomProps({
    tableInstance: tableInstance
  }) : muiTableToolbarBottomProps;
  var stackAlertBanner = isMobile || positionToolbarAlertBanner === 'bottom' && positionToolbarActions === 'bottom' || positionToolbarAlertBanner === 'bottom' && !!renderToolbarCustomActions && positionToolbarActions === 'bottom';
  return React.createElement(Toolbar, Object.assign({
    id: "mrt-" + idPrefix + "-toolbar-bottom",
    variant: "dense"
  }, toolbarProps, {
    sx: function sx(theme) {
      return _extends({}, commonToolbarStyles({
        theme: theme
      }), {
        bottom: isFullScreen ? '0' : undefined,
        position: isFullScreen ? 'fixed' : undefined,
        boxShadow: "-3px 0 6px " + alpha(theme.palette.common.black, 0.1)
      }, toolbarProps == null ? void 0 : toolbarProps.sx);
    }
  }), React.createElement(MRT_LinearProgressBar, {
    tableInstance: tableInstance
  }), positionToolbarAlertBanner === 'bottom' && React.createElement(MRT_ToolbarAlertBanner, {
    tableInstance: tableInstance
  }), React.createElement(Box, {
    sx: {
      display: 'flex',
      justifyContent: 'space-between',
      width: '100%',
      position: stackAlertBanner ? 'relative' : 'absolute',
      right: 0,
      top: 0
    }
  }, enableToolbarInternalActions && positionToolbarActions === 'bottom' ? React.createElement(MRT_ToolbarInternalButtons, {
    tableInstance: tableInstance
  }) : React.createElement("span", null), enablePagination && ['bottom', 'both'].includes(positionPagination != null ? positionPagination : '') && React.createElement(MRT_TablePagination, {
    tableInstance: tableInstance
  })));
};

var MRT_FilterTextField = function MRT_FilterTextField(_ref) {
  var _localization$filterB, _localization$filterM, _localization$, _localization$clearFi, _column$filterSelectO;

  var header = _ref.header,
      inputIndex = _ref.inputIndex,
      tableInstance = _ref.tableInstance;
  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      _tableInstance$option2 = _tableInstance$option.icons,
      FilterListIcon = _tableInstance$option2.FilterListIcon,
      CloseIcon = _tableInstance$option2.CloseIcon,
      idPrefix = _tableInstance$option.idPrefix,
      localization = _tableInstance$option.localization,
      muiTableHeadCellFilterTextFieldProps = _tableInstance$option.muiTableHeadCellFilterTextFieldProps,
      setCurrentFilterFns = tableInstance.setCurrentFilterFns;
  var column = header.column;

  var _getState = getState(),
      currentFilterFns = _getState.currentFilterFns;

  var _useState = useState(null),
      anchorEl = _useState[0],
      setAnchorEl = _useState[1];

  var mTableHeadCellFilterTextFieldProps = muiTableHeadCellFilterTextFieldProps instanceof Function ? muiTableHeadCellFilterTextFieldProps({
    column: column,
    tableInstance: tableInstance
  }) : muiTableHeadCellFilterTextFieldProps;
  var mcTableHeadCellFilterTextFieldProps = column.muiTableHeadCellFilterTextFieldProps instanceof Function ? column.muiTableHeadCellFilterTextFieldProps({
    column: column,
    tableInstance: tableInstance
  }) : column.muiTableHeadCellFilterTextFieldProps;

  var textFieldProps = _extends({}, mTableHeadCellFilterTextFieldProps, mcTableHeadCellFilterTextFieldProps);

  var _useState2 = useState(function () {
    var _column$getFilterValu, _column$getFilterValu2, _column$getFilterValu3;

    return inputIndex !== undefined ? (_column$getFilterValu = (_column$getFilterValu2 = column.getFilterValue()) == null ? void 0 : _column$getFilterValu2[inputIndex]) != null ? _column$getFilterValu : '' : (_column$getFilterValu3 = column.getFilterValue()) != null ? _column$getFilterValu3 : '';
  }),
      filterValue = _useState2[0],
      setFilterValue = _useState2[1];

  var handleChangeDebounced = useCallback(debounce(function (event) {
    var _event$target$value;

    return inputIndex !== undefined ? column.setFilterValue(function (old) {
      var newFilterValues = old != null ? old : ['', ''];
      newFilterValues[inputIndex] = event.target.value;
      return newFilterValues;
    }) : column.setFilterValue((_event$target$value = event.target.value) != null ? _event$target$value : undefined);
  }, 150), []);

  var handleFilterMenuOpen = function handleFilterMenuOpen(event) {
    setAnchorEl(event.currentTarget);
  };

  var handleClear = function handleClear() {
    setFilterValue('');

    if (inputIndex !== undefined) {
      column.setFilterValue(function (old) {
        var newFilterValues = old != null ? old : ['', ''];
        newFilterValues[inputIndex] = undefined;
        return newFilterValues;
      });
    } else {
      column.setFilterValue(undefined);
    }
  };

  var handleClearFilterChip = function handleClearFilterChip() {
    setFilterValue('');
    column.setFilterValue(undefined);
    setCurrentFilterFns(function (prev) {
      var _extends2;

      return _extends({}, prev, (_extends2 = {}, _extends2[header.id] = MRT_FILTER_OPTION.FUZZY, _extends2));
    });
  };

  if (column.Filter) {
    return React.createElement(React.Fragment, null, column.Filter == null ? void 0 : column.Filter({
      header: header,
      tableInstance: tableInstance
    }));
  }

  var filterId = "mrt-" + idPrefix + "-" + header.id + "-filter-text-field" + (inputIndex != null ? inputIndex : '');
  var filterFn = currentFilterFns == null ? void 0 : currentFilterFns[header.id];
  var isSelectFilter = !!column.filterSelectOptions;
  var filterChipLabel = !(filterFn instanceof Function) && [MRT_FILTER_OPTION.EMPTY, MRT_FILTER_OPTION.NOT_EMPTY].includes(filterFn) ? //@ts-ignore
  localization["filter" + (filterFn.charAt(0).toUpperCase() + filterFn.slice(1))] : '';
  var filterPlaceholder = inputIndex === undefined ? (_localization$filterB = localization.filterByColumn) == null ? void 0 : _localization$filterB.replace('{column}', String(column.columnDef.header)) : inputIndex === 0 ? localization.min : inputIndex === 1 ? localization.max : '';
  return React.createElement(React.Fragment, null, React.createElement(TextField, Object.assign({
    fullWidth: true,
    id: filterId,
    inputProps: {
      disabled: !!filterChipLabel,
      sx: {
        textOverflow: 'ellipsis',
        width: filterChipLabel ? 0 : undefined
      },
      title: filterPlaceholder
    },
    helperText: !inputIndex ? React.createElement("label", {
      htmlFor: filterId
    }, filterFn instanceof Function ? (_localization$filterM = localization.filterMode.replace('{filterType}', // @ts-ignore
    (_localization$ = localization["filter" + (filterFn.name.charAt(0).toUpperCase() + filterFn.name.slice(1))]) != null ? _localization$ : '')) != null ? _localization$filterM : '' : localization.filterMode.replace('{filterType}', // @ts-ignore
    localization["filter" + (filterFn.charAt(0).toUpperCase() + filterFn.slice(1))])) : null,
    FormHelperTextProps: {
      sx: {
        fontSize: '0.6rem',
        lineHeight: '0.8rem',
        whiteSpace: 'nowrap'
      }
    },
    label: isSelectFilter && !filterValue ? filterPlaceholder : undefined,
    margin: "none",
    placeholder: filterChipLabel || isSelectFilter ? undefined : filterPlaceholder,
    onChange: function onChange(event) {
      setFilterValue(event.target.value);
      handleChangeDebounced(event);
    },
    onClick: function onClick(e) {
      return e.stopPropagation();
    },
    select: isSelectFilter,
    value: filterValue != null ? filterValue : '',
    variant: "standard",
    InputProps: {
      startAdornment: !isSelectFilter && !inputIndex && React.createElement(InputAdornment, {
        position: "start"
      }, React.createElement(Tooltip, {
        arrow: true,
        title: localization.changeFilterMode
      }, React.createElement("span", null, React.createElement(IconButton, {
        "aria-label": localization.changeFilterMode,
        onClick: handleFilterMenuOpen,
        size: "small",
        sx: {
          height: '1.75rem',
          width: '1.75rem'
        }
      }, React.createElement(FilterListIcon, null)))), filterChipLabel && React.createElement(Chip, {
        onDelete: handleClearFilterChip,
        label: filterChipLabel
      })),
      endAdornment: !filterChipLabel && React.createElement(InputAdornment, {
        position: "end"
      }, React.createElement(Tooltip, {
        arrow: true,
        disableHoverListener: isSelectFilter,
        placement: "right",
        title: (_localization$clearFi = localization.clearFilter) != null ? _localization$clearFi : ''
      }, React.createElement("span", null, React.createElement(IconButton, {
        "aria-label": localization.clearFilter,
        disabled: !(filterValue != null && filterValue.length),
        onClick: handleClear,
        size: "small",
        sx: {
          height: '1.75rem',
          width: '1.75rem'
        }
      }, React.createElement(CloseIcon, null)))))
    }
  }, textFieldProps, {
    sx: _extends({
      m: '-0.25rem',
      p: 0,
      minWidth: !filterChipLabel ? '6rem' : 'auto',
      width: 'calc(100% + 0.5rem)',
      mt: isSelectFilter && !filterValue ? '-1rem' : undefined,
      '&	.MuiSelect-icon': {
        mr: '1.5rem'
      }
    }, textFieldProps == null ? void 0 : textFieldProps.sx)
  }), isSelectFilter && React.createElement(MenuItem, {
    divider: true,
    disabled: !filterValue,
    value: ""
  }, localization.clearFilter), column == null ? void 0 : (_column$filterSelectO = column.filterSelectOptions) == null ? void 0 : _column$filterSelectO.map(function (option) {
    var value;
    var text;

    if (typeof option === 'string') {
      value = option;
      text = option;
    } else if (typeof option === 'object') {
      value = option.value;
      text = option.text;
    }

    return React.createElement(MenuItem, {
      key: value,
      value: value
    }, text);
  })), React.createElement(MRT_FilterOptionMenu, {
    anchorEl: anchorEl,
    header: header,
    setAnchorEl: setAnchorEl,
    tableInstance: tableInstance
  }));
};

var MRT_ToggleColumnActionMenuButton = function MRT_ToggleColumnActionMenuButton(_ref) {
  var header = _ref.header,
      tableInstance = _ref.tableInstance;
  var _tableInstance$option = tableInstance.options,
      MoreVertIcon = _tableInstance$option.icons.MoreVertIcon,
      localization = _tableInstance$option.localization,
      muiTableHeadCellColumnActionsButtonProps = _tableInstance$option.muiTableHeadCellColumnActionsButtonProps;
  var column = header.column;

  var _useState = useState(null),
      anchorEl = _useState[0],
      setAnchorEl = _useState[1];

  var handleClick = function handleClick(event) {
    event.stopPropagation();
    event.preventDefault();
    setAnchorEl(event.currentTarget);
  };

  var mTableHeadCellColumnActionsButtonProps = muiTableHeadCellColumnActionsButtonProps instanceof Function ? muiTableHeadCellColumnActionsButtonProps({
    column: column,
    tableInstance: tableInstance
  }) : muiTableHeadCellColumnActionsButtonProps;
  var mcTableHeadCellColumnActionsButtonProps = column.muiTableHeadCellColumnActionsButtonProps instanceof Function ? column.muiTableHeadCellColumnActionsButtonProps({
    column: column,
    tableInstance: tableInstance
  }) : column.muiTableHeadCellColumnActionsButtonProps;

  var iconButtonProps = _extends({}, mTableHeadCellColumnActionsButtonProps, mcTableHeadCellColumnActionsButtonProps);

  return React.createElement(React.Fragment, null, React.createElement(Tooltip, {
    arrow: true,
    enterDelay: 1000,
    enterNextDelay: 1000,
    placement: "top",
    title: localization.columnActions
  }, React.createElement(IconButton, Object.assign({
    "aria-label": localization.columnActions,
    onClick: handleClick,
    size: "small"
  }, iconButtonProps, {
    sx: _extends({
      height: '2rem',
      mr: '2px',
      mt: '-0.2rem',
      opacity: 0.5,
      transition: 'opacity 0.2s',
      width: '2rem',
      '&:hover': {
        opacity: 1
      }
    }, iconButtonProps.sx)
  }), React.createElement(MoreVertIcon, null))), React.createElement(MRT_ColumnActionMenu, {
    anchorEl: anchorEl,
    header: header,
    setAnchorEl: setAnchorEl,
    tableInstance: tableInstance
  }));
};

var MRT_FilterRangeFields = function MRT_FilterRangeFields(_ref) {
  var header = _ref.header,
      tableInstance = _ref.tableInstance;
  var localization = tableInstance.options.localization;
  return React.createElement(Box, {
    sx: {
      display: 'grid',
      gridTemplateColumns: '6fr auto 5fr'
    }
  }, React.createElement(MRT_FilterTextField, {
    header: header,
    inputIndex: 0,
    tableInstance: tableInstance
  }), React.createElement(Box, {
    sx: {
      width: '100%',
      minWidth: '5ch',
      textAlign: 'center'
    }
  }, localization.to), React.createElement(MRT_FilterTextField, {
    header: header,
    inputIndex: 1,
    tableInstance: tableInstance
  }));
};

var MRT_TableHeadCell = function MRT_TableHeadCell(_ref) {
  var _getState2, _getState2$currentFil, _column$columnDef$Hea, _column$columnDef, _column$minSize, _column$columnDef$hea, _column$columnDef$hea2;

  var header = _ref.header,
      tableInstance = _ref.tableInstance;
  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      enableColumnActions = _tableInstance$option.enableColumnActions,
      enableColumnFilters = _tableInstance$option.enableColumnFilters,
      enableColumnResizing = _tableInstance$option.enableColumnResizing,
      _tableInstance$option2 = _tableInstance$option.icons,
      FilterAltIcon = _tableInstance$option2.FilterAltIcon,
      FilterAltOff = _tableInstance$option2.FilterAltOff,
      localization = _tableInstance$option.localization,
      muiTableHeadCellProps = _tableInstance$option.muiTableHeadCellProps,
      setShowFilters = tableInstance.setShowFilters;

  var _getState = getState(),
      currentFilterFns = _getState.currentFilterFns,
      isDensePadding = _getState.isDensePadding,
      showFilters = _getState.showFilters;

  var column = header.column;
  var mTableHeadCellProps = muiTableHeadCellProps instanceof Function ? muiTableHeadCellProps({
    column: column,
    tableInstance: tableInstance
  }) : muiTableHeadCellProps;
  var mcTableHeadCellProps = column.muiTableHeadCellProps instanceof Function ? column.muiTableHeadCellProps({
    column: column,
    tableInstance: tableInstance
  }) : column.muiTableHeadCellProps;

  var tableCellProps = _extends({}, mTableHeadCellProps, mcTableHeadCellProps);

  var sortTooltip = !!column.getIsSorted() ? column.getIsSorted() === 'desc' ? localization.sortedByColumnDesc.replace('{column}', column.columnDef.header) : localization.sortedByColumnAsc.replace('{column}', column.columnDef.header) : localization.unsorted;
  var filterFn = (_getState2 = getState()) == null ? void 0 : (_getState2$currentFil = _getState2.currentFilterFns) == null ? void 0 : _getState2$currentFil[header.id];
  var filterTooltip = !!column.getFilterValue() ? localization.filteringByColumn.replace('{column}', String(column.columnDef.header)).replace('{filterType}', filterFn instanceof Function ? '' : // @ts-ignore
  localization["filter" + (filterFn.charAt(0).toUpperCase() + filterFn.slice(1))]).replace('{filterValue}', "\"" + (Array.isArray(column.getFilterValue()) ? column.getFilterValue().join("\" " + localization.and + " \"") : column.getFilterValue()) + "\"").replace('" "', '') : localization.showHideFilters;
  var headerElement = (_column$columnDef$Hea = (_column$columnDef = column.columnDef) == null ? void 0 : _column$columnDef.Header == null ? void 0 : _column$columnDef.Header({
    header: header,
    tableInstance: tableInstance
  })) != null ? _column$columnDef$Hea : header.renderHeader();

  var getIsLastLeftPinnedColumn = function getIsLastLeftPinnedColumn() {
    return column.getIsPinned() === 'left' && tableInstance.getLeftLeafHeaders().length - 1 === column.getPinnedIndex();
  };

  var getIsFirstRightPinnedColumn = function getIsFirstRightPinnedColumn() {
    return column.getIsPinned() === 'right' && column.getPinnedIndex() === 0;
  };

  var getTotalRight = function getTotalRight() {
    return (tableInstance.getRightLeafHeaders().length - 1 - column.getPinnedIndex()) * 150;
  };

  return React.createElement(TableCell, Object.assign({
    align: column.columnDefType === 'group' ? 'center' : 'left',
    colSpan: header.colSpan
  }, tableCellProps, {
    sx: function sx(theme) {
      return _extends({
        backgroundColor: column.getIsPinned() && column.columnDefType !== 'group' ? alpha(lighten(theme.palette.background["default"], 0.04), 0.95) : 'inherit',
        backgroundImage: 'inherit',
        boxShadow: getIsLastLeftPinnedColumn() ? "4px 0 4px -2px " + alpha(theme.palette.common.black, 0.1) : getIsFirstRightPinnedColumn() ? "-4px 0 4px -2px " + alpha(theme.palette.common.black, 0.1) : undefined,
        fontWeight: 'bold',
        height: '100%',
        left: column.getIsPinned() === 'left' ? column.getStart('left') + "px" : undefined,
        overflow: 'visible',
        p: isDensePadding ? column.columnDefType === 'display' ? '0 0.5rem' : '0.5rem' : column.columnDefType === 'display' ? '0.5rem 0.75rem' : '1rem',
        pb: column.columnDefType === 'display' ? 0 : undefined,
        position: column.getIsPinned() && column.columnDefType !== 'group' ? 'sticky' : undefined,
        pt: column.columnDefType === 'display' ? 0 : isDensePadding ? '0.75rem' : '1.25rem',
        right: column.getIsPinned() === 'right' ? getTotalRight() + "px" : undefined,
        transition: "all " + (enableColumnResizing ? 0 : '0.2s') + " ease-in-out",
        verticalAlign: 'text-top',
        zIndex: column.getIsResizing() ? 3 : column.getIsPinned() && column.columnDefType !== 'group' ? 2 : 1
      }, tableCellProps == null ? void 0 : tableCellProps.sx);
    },
    style: {
      maxWidth: "min(" + column.getSize() + "px, fit-content)",
      minWidth: "max(" + column.getSize() + "px, " + ((_column$minSize = column.minSize) != null ? _column$minSize : 30) + "px)",
      width: header.getSize()
    }
  }), header.isPlaceholder ? null : column.columnDefType === 'display' ? headerElement : React.createElement(Box, {
    sx: {
      alignItems: 'flex-start',
      display: 'flex',
      justifyContent: column.columnDefType === 'group' ? 'center' : 'space-between',
      width: '100%'
    }
  }, React.createElement(Box, {
    onClick: function onClick() {
      return column.toggleSorting();
    },
    sx: {
      alignItems: 'center',
      cursor: column.getCanSort() && column.columnDefType !== 'group' ? 'pointer' : undefined,
      display: 'flex',
      flexWrap: 'nowrap',
      whiteSpace: ((_column$columnDef$hea = (_column$columnDef$hea2 = column.columnDef.header) == null ? void 0 : _column$columnDef$hea2.length) != null ? _column$columnDef$hea : 0) < 24 ? 'nowrap' : 'normal'
    }
  }, headerElement, column.columnDefType === 'data' && column.getCanSort() && React.createElement(Tooltip, {
    arrow: true,
    placement: "top",
    title: sortTooltip
  }, React.createElement(TableSortLabel, {
    "aria-label": sortTooltip,
    active: !!column.getIsSorted(),
    direction: column.getIsSorted() ? column.getIsSorted() : undefined
  })), column.columnDefType === 'data' && enableColumnFilters && !!column.getCanFilter() && React.createElement(Tooltip, {
    arrow: true,
    placement: "top",
    title: filterTooltip
  }, React.createElement(IconButton, {
    disableRipple: true,
    onClick: function onClick(event) {
      event.stopPropagation();
      setShowFilters(!showFilters);
    },
    size: "small",
    sx: {
      m: 0,
      opacity: !!column.getFilterValue() ? 0.8 : 0,
      p: '2px',
      transition: 'all 0.2s ease-in-out',
      '&:hover': {
        backgroundColor: 'transparent',
        opacity: 0.8
      }
    }
  }, showFilters && !column.getFilterValue() ? React.createElement(FilterAltOff, null) : React.createElement(FilterAltIcon, null)))), (enableColumnActions || column.enableColumnActions) && column.enableColumnActions !== false && column.columnDefType !== 'group' && React.createElement(MRT_ToggleColumnActionMenuButton, {
    header: header,
    tableInstance: tableInstance
  }), column.getCanResize() && React.createElement(Divider, {
    flexItem: true,
    orientation: "vertical",
    onDoubleClick: function onDoubleClick() {
      return column.resetSize();
    },
    sx: function sx(theme) {
      return {
        borderRadius: '2px',
        borderRightWidth: '2px',
        cursor: 'col-resize',
        height: showFilters && column.columnDefType === 'data' ? '4rem' : '2rem',
        opacity: 0.8,
        position: 'absolute',
        right: '1px',
        touchAction: 'none',
        transition: column.getIsResizing() ? undefined : 'all 0.2s ease-in-out',
        userSelect: 'none',
        zIndex: 2000,
        '&:active': {
          backgroundColor: theme.palette.info.main,
          opacity: 1
        }
      };
    },
    onMouseDown: header.getResizeHandler(),
    onTouchStart: header.getResizeHandler(),
    style: {
      transform: column.getIsResizing() ? "translateX(" + getState().columnSizingInfo.deltaOffset + "px)" : 'none'
    }
  })), column.columnDefType === 'data' && column.getCanFilter() && React.createElement(Collapse, {
    "in": showFilters,
    mountOnEnter: true,
    unmountOnExit: true
  }, currentFilterFns[column.id] === MRT_FILTER_OPTION.BETWEEN ? React.createElement(MRT_FilterRangeFields, {
    header: header,
    tableInstance: tableInstance
  }) : React.createElement(MRT_FilterTextField, {
    header: header,
    tableInstance: tableInstance
  })));
};

var MRT_TableHeadRow = function MRT_TableHeadRow(_ref) {
  var headerGroup = _ref.headerGroup,
      tableInstance = _ref.tableInstance;
  var muiTableHeadRowProps = tableInstance.options.muiTableHeadRowProps;
  var tableRowProps = muiTableHeadRowProps instanceof Function ? muiTableHeadRowProps({
    headerGroup: headerGroup,
    tableInstance: tableInstance
  }) : muiTableHeadRowProps;
  return React.createElement(TableRow, Object.assign({}, tableRowProps, {
    sx: function sx(theme) {
      return _extends({
        boxShadow: "4px 0 8px " + alpha(theme.palette.common.black, 0.1),
        backgroundColor: lighten(theme.palette.background["default"], 0.04)
      }, tableRowProps == null ? void 0 : tableRowProps.sx);
    }
  }), headerGroup.headers.map(function (header, index) {
    return React.createElement(MRT_TableHeadCell, {
      header: header,
      key: header.id || index,
      tableInstance: tableInstance
    });
  }));
};

var MRT_TableHead = function MRT_TableHead(_ref) {
  var tableInstance = _ref.tableInstance;
  var getHeaderGroups = tableInstance.getHeaderGroups,
      muiTableHeadProps = tableInstance.options.muiTableHeadProps;
  var tableHeadProps = muiTableHeadProps instanceof Function ? muiTableHeadProps({
    tableInstance: tableInstance
  }) : muiTableHeadProps;
  return React.createElement(TableHead, Object.assign({}, tableHeadProps), getHeaderGroups().map(function (headerGroup) {
    return React.createElement(MRT_TableHeadRow, {
      headerGroup: headerGroup,
      key: headerGroup.id,
      tableInstance: tableInstance
    });
  }));
};

var MRT_EditCellTextField = function MRT_EditCellTextField(_ref) {
  var cell = _ref.cell,
      tableInstance = _ref.tableInstance;
  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      idPrefix = _tableInstance$option.idPrefix,
      enableEditing = _tableInstance$option.enableEditing,
      muiTableBodyCellEditTextFieldProps = _tableInstance$option.muiTableBodyCellEditTextFieldProps,
      onCellEditBlur = _tableInstance$option.onCellEditBlur,
      onCellEditChange = _tableInstance$option.onCellEditChange,
      setCurrentEditingCell = tableInstance.setCurrentEditingCell,
      setCurrentEditingRow = tableInstance.setCurrentEditingRow;

  var _useState = useState(cell.getValue()),
      value = _useState[0],
      setValue = _useState[1];

  var column = cell.column,
      row = cell.row;

  var handleChange = function handleChange(event) {
    setValue(event.target.value);
    column.onCellEditChange == null ? void 0 : column.onCellEditChange({
      event: event,
      cell: cell,
      tableInstance: tableInstance
    });
    onCellEditChange == null ? void 0 : onCellEditChange({
      event: event,
      cell: cell,
      tableInstance: tableInstance
    });
  };

  var handleBlur = function handleBlur(event) {
    if (getState().currentEditingRow) {
      row._valuesCache[column.id] = value;
      setCurrentEditingRow(_extends({}, getState().currentEditingRow));
    }

    setCurrentEditingCell(null);
    column.onCellEditBlur == null ? void 0 : column.onCellEditBlur({
      event: event,
      cell: cell,
      tableInstance: tableInstance
    });
    onCellEditBlur == null ? void 0 : onCellEditBlur({
      event: event,
      cell: cell,
      tableInstance: tableInstance
    });
  };

  var mTableBodyCellEditTextFieldProps = muiTableBodyCellEditTextFieldProps instanceof Function ? muiTableBodyCellEditTextFieldProps({
    cell: cell,
    tableInstance: tableInstance
  }) : muiTableBodyCellEditTextFieldProps;
  var mcTableBodyCellEditTextFieldProps = column.muiTableBodyCellEditTextFieldProps instanceof Function ? column.muiTableBodyCellEditTextFieldProps({
    cell: cell,
    tableInstance: tableInstance
  }) : column.muiTableBodyCellEditTextFieldProps;

  var textFieldProps = _extends({}, mTableBodyCellEditTextFieldProps, mcTableBodyCellEditTextFieldProps);

  if (enableEditing && column.enableEditing !== false && column.Edit) {
    return React.createElement(React.Fragment, null, column.Edit == null ? void 0 : column.Edit({
      cell: cell,
      tableInstance: tableInstance
    }));
  }

  return React.createElement(TextField, Object.assign({
    id: "mrt-" + idPrefix + "-edit-cell-text-field-" + cell.id,
    margin: "dense",
    onBlur: handleBlur,
    onChange: handleChange,
    onClick: function onClick(e) {
      return e.stopPropagation();
    },
    placeholder: column.columnDef.header,
    value: value,
    variant: "standard"
  }, textFieldProps));
};

var MRT_CopyButton = function MRT_CopyButton(_ref) {
  var cell = _ref.cell,
      children = _ref.children,
      tableInstance = _ref.tableInstance;
  var _tableInstance$option = tableInstance.options,
      localization = _tableInstance$option.localization,
      muiTableBodyCellCopyButtonProps = _tableInstance$option.muiTableBodyCellCopyButtonProps;

  var _useState = useState(false),
      copied = _useState[0],
      setCopied = _useState[1];

  var handleCopy = function handleCopy(text) {
    navigator.clipboard.writeText(text);
    setCopied(true);
    setTimeout(function () {
      return setCopied(false);
    }, 4000);
  };

  var mTableBodyCellCopyButtonProps = muiTableBodyCellCopyButtonProps instanceof Function ? muiTableBodyCellCopyButtonProps({
    cell: cell,
    tableInstance: tableInstance
  }) : muiTableBodyCellCopyButtonProps;
  var mcTableBodyCellCopyButtonProps = cell.column.muiTableBodyCellCopyButtonProps instanceof Function ? cell.column.muiTableBodyCellCopyButtonProps({
    cell: cell,
    tableInstance: tableInstance
  }) : cell.column.muiTableBodyCellCopyButtonProps;

  var buttonProps = _extends({}, mTableBodyCellCopyButtonProps, mcTableBodyCellCopyButtonProps);

  return React.createElement(Tooltip, {
    arrow: true,
    enterDelay: 1000,
    enterNextDelay: 1000,
    placement: "top",
    title: copied ? localization.copiedToClipboard : localization.clickToCopy
  }, React.createElement(Button, Object.assign({
    "aria-label": localization.clickToCopy,
    onClick: function onClick() {
      return handleCopy(cell.getValue());
    },
    size: "small",
    type: "button",
    variant: "text"
  }, buttonProps, {
    sx: _extends({
      backgroundColor: 'transparent',
      border: 'none',
      color: 'inherit',
      cursor: 'copy',
      fontFamily: 'inherit',
      fontSize: 'inherit',
      letterSpacing: 'inherit',
      m: '-0.25rem',
      minWidth: 'unset',
      textAlign: 'inherit',
      textTransform: 'inherit'
    }, buttonProps == null ? void 0 : buttonProps.sx)
  }), children));
};

var MRT_TableBodyCell = function MRT_TableBodyCell(_ref) {
  var _column$minSize, _cell$column$columnDe, _cell$column$columnDe2, _row$subRows, _cell$column$columnDe3, _cell$column$columnDe4, _row$subRows$length, _row$subRows2;

  var cell = _ref.cell,
      enableHover = _ref.enableHover,
      tableInstance = _ref.tableInstance;
  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      editingMode = _tableInstance$option.editingMode,
      enableClickToCopy = _tableInstance$option.enableClickToCopy,
      enableEditing = _tableInstance$option.enableEditing,
      idPrefix = _tableInstance$option.idPrefix,
      muiTableBodyCellProps = _tableInstance$option.muiTableBodyCellProps,
      muiTableBodyCellSkeletonProps = _tableInstance$option.muiTableBodyCellSkeletonProps,
      onCellClick = _tableInstance$option.onCellClick,
      setCurrentEditingCell = tableInstance.setCurrentEditingCell;

  var _getState = getState(),
      currentEditingCell = _getState.currentEditingCell,
      currentEditingRow = _getState.currentEditingRow,
      isDensePadding = _getState.isDensePadding,
      isLoading = _getState.isLoading,
      showSkeletons = _getState.showSkeletons;

  var column = cell.column,
      row = cell.row;
  var mTableCellBodyProps = muiTableBodyCellProps instanceof Function ? muiTableBodyCellProps({
    cell: cell,
    tableInstance: tableInstance
  }) : muiTableBodyCellProps;
  var mcTableCellBodyProps = column.muiTableBodyCellProps instanceof Function ? column.muiTableBodyCellProps({
    cell: cell,
    tableInstance: tableInstance
  }) : column.muiTableBodyCellProps;

  var tableCellProps = _extends({}, mTableCellBodyProps, mcTableCellBodyProps);

  var skeletonWidth = useMemo(function () {
    return column.columnDefType === 'display' ? column.getSize() / 2 : Math.random() * (column.getSize() - column.getSize() / 3) + column.getSize() / 3;
  }, [column.columnDefType, column.getSize()]);
  var isEditable = (enableEditing || column.enableEditing) && column.enableEditing !== false;
  var isEditing = isEditable && (editingMode === 'table' || (currentEditingRow == null ? void 0 : currentEditingRow.id) === row.id || (currentEditingCell == null ? void 0 : currentEditingCell.id) === cell.id);

  var handleDoubleClick = function handleDoubleClick(_event) {
    if ((enableEditing || column.enableEditing) && column.enableEditing !== false && editingMode === 'cell') {
      setCurrentEditingCell(cell);
      setTimeout(function () {
        var textField = document.getElementById("mrt-" + idPrefix + "-edit-cell-text-field-" + cell.id);

        if (textField) {
          textField.focus();
          textField.select();
        }
      }, 200);
    }
  };

  var getIsLastLeftPinnedColumn = function getIsLastLeftPinnedColumn() {
    return column.getIsPinned() === 'left' && tableInstance.getLeftLeafHeaders().length - 1 === column.getPinnedIndex();
  };

  var getIsFirstRightPinnedColumn = function getIsFirstRightPinnedColumn() {
    return column.getIsPinned() === 'right' && column.getPinnedIndex() === 0;
  };

  var getTotalRight = function getTotalRight() {
    return (tableInstance.getRightLeafHeaders().length - 1 - column.getPinnedIndex()) * 150;
  };

  return React.createElement(TableCell, Object.assign({
    onClick: function onClick(event) {
      return onCellClick == null ? void 0 : onCellClick({
        event: event,
        cell: cell,
        tableInstance: tableInstance
      });
    },
    onDoubleClick: handleDoubleClick
  }, tableCellProps, {
    sx: function sx(theme) {
      return _extends({
        backgroundColor: column.getIsPinned() ? alpha(lighten(theme.palette.background["default"], 0.04), 0.95) : undefined,
        boxShadow: getIsLastLeftPinnedColumn() ? "4px 0 4px -2px " + alpha(theme.palette.common.black, 0.1) : getIsFirstRightPinnedColumn() ? "-4px 0 4px -2px " + alpha(theme.palette.common.black, 0.1) : undefined,
        cursor: isEditable && editingMode === 'cell' ? 'pointer' : 'text',
        left: column.getIsPinned() === 'left' ? column.getStart('left') + "px" : undefined,
        p: isDensePadding ? column.columnDefType === 'display' ? '0 0.5rem' : '0.5rem' : column.columnDefType === 'display' ? '0.5rem 0.75rem' : '1rem',
        pl: column.id === 'mrt-expand' ? row.depth + (isDensePadding ? 0.5 : 0.75) + "rem" : undefined,
        position: column.getIsPinned() ? 'sticky' : 'relative',
        right: column.getIsPinned() === 'right' ? getTotalRight() + "px" : undefined,
        transition: 'all 0.2s ease-in-out',
        whiteSpace: isDensePadding ? 'nowrap' : 'normal',
        zIndex: column.getIsPinned() ? 1 : undefined,
        '&:hover': {
          backgroundColor: enableHover ? theme.palette.mode === 'dark' ? lighten(theme.palette.background["default"], 0.13) + " !important" : darken(theme.palette.background["default"], 0.07) + " !important" : undefined
        }
      }, tableCellProps == null ? void 0 : tableCellProps.sx);
    },
    style: {
      maxWidth: "min(" + column.getSize() + "px, fit-content)",
      minWidth: "max(" + column.getSize() + "px, " + ((_column$minSize = column.minSize) != null ? _column$minSize : 30) + "px)",
      width: column.getSize()
    }
  }), React.createElement(React.Fragment, null, isLoading || showSkeletons ? React.createElement(Skeleton, Object.assign({
    animation: "wave",
    height: 20,
    width: skeletonWidth
  }, muiTableBodyCellSkeletonProps)) : column.columnDefType === 'display' ? column.columnDef.Cell == null ? void 0 : column.columnDef.Cell({
    cell: cell,
    tableInstance: tableInstance
  }) : cell.getIsPlaceholder() || row.getIsGrouped() && column.id !== row.groupingColumnId ? null : cell.getIsAggregated() ? cell.renderAggregatedCell() : isEditing ? React.createElement(MRT_EditCellTextField, {
    cell: cell,
    tableInstance: tableInstance
  }) : (enableClickToCopy || column.enableClickToCopy) && column.enableClickToCopy !== false ? React.createElement(React.Fragment, null, React.createElement(MRT_CopyButton, {
    cell: cell,
    tableInstance: tableInstance
  }, React.createElement(React.Fragment, null, (_cell$column$columnDe = (_cell$column$columnDe2 = cell.column.columnDef) == null ? void 0 : _cell$column$columnDe2.Cell == null ? void 0 : _cell$column$columnDe2.Cell({
    cell: cell,
    tableInstance: tableInstance
  })) != null ? _cell$column$columnDe : cell.renderCell())), row.getIsGrouped() && React.createElement(React.Fragment, null, " (", (_row$subRows = row.subRows) == null ? void 0 : _row$subRows.length, ")")) : React.createElement(React.Fragment, null, (_cell$column$columnDe3 = (_cell$column$columnDe4 = cell.column.columnDef) == null ? void 0 : _cell$column$columnDe4.Cell == null ? void 0 : _cell$column$columnDe4.Cell({
    cell: cell,
    tableInstance: tableInstance
  })) != null ? _cell$column$columnDe3 : cell.renderCell(), row.getIsGrouped() && React.createElement(React.Fragment, null, " (", (_row$subRows$length = (_row$subRows2 = row.subRows) == null ? void 0 : _row$subRows2.length) != null ? _row$subRows$length : '', ")"))));
};

var MRT_TableDetailPanel = function MRT_TableDetailPanel(_ref) {
  var row = _ref.row,
      tableInstance = _ref.tableInstance;
  var getVisibleFlatColumns = tableInstance.getVisibleFlatColumns,
      _tableInstance$option = tableInstance.options,
      muiTableBodyRowProps = _tableInstance$option.muiTableBodyRowProps,
      muiTableDetailPanelProps = _tableInstance$option.muiTableDetailPanelProps,
      onDetailPanelClick = _tableInstance$option.onDetailPanelClick,
      renderDetailPanel = _tableInstance$option.renderDetailPanel;
  var tableRowProps = muiTableBodyRowProps instanceof Function ? muiTableBodyRowProps({
    row: row,
    tableInstance: tableInstance
  }) : muiTableBodyRowProps;
  var tableCellProps = muiTableDetailPanelProps instanceof Function ? muiTableDetailPanelProps({
    row: row,
    tableInstance: tableInstance
  }) : muiTableDetailPanelProps;
  return React.createElement(TableRow, Object.assign({}, tableRowProps), React.createElement(TableCell, Object.assign({
    colSpan: getVisibleFlatColumns().length + 10,
    onClick: function onClick(event) {
      return onDetailPanelClick == null ? void 0 : onDetailPanelClick({
        event: event,
        row: row,
        tableInstance: tableInstance
      });
    }
  }, tableCellProps, {
    sx: _extends({
      borderBottom: !row.getIsExpanded() ? 'none' : undefined,
      pb: row.getIsExpanded() ? '1rem' : 0,
      pt: row.getIsExpanded() ? '1rem' : 0,
      transition: 'all 0.2s ease-in-out'
    }, tableCellProps == null ? void 0 : tableCellProps.sx)
  }), React.createElement(Collapse, {
    "in": row.getIsExpanded()
  }, renderDetailPanel == null ? void 0 : renderDetailPanel({
    row: row,
    tableInstance: tableInstance
  }))));
};

var MRT_TableBodyRow = function MRT_TableBodyRow(_ref) {
  var row = _ref.row,
      tableInstance = _ref.tableInstance;
  var _tableInstance$option = tableInstance.options,
      muiTableBodyRowProps = _tableInstance$option.muiTableBodyRowProps,
      onRowClick = _tableInstance$option.onRowClick,
      renderDetailPanel = _tableInstance$option.renderDetailPanel;
  var tableRowProps = muiTableBodyRowProps instanceof Function ? muiTableBodyRowProps({
    row: row,
    tableInstance: tableInstance
  }) : muiTableBodyRowProps;
  return React.createElement(React.Fragment, null, React.createElement(TableRow, Object.assign({
    hover: true,
    onClick: function onClick(event) {
      return onRowClick == null ? void 0 : onRowClick({
        event: event,
        row: row,
        tableInstance: tableInstance
      });
    },
    selected: row.getIsSelected()
  }, tableRowProps, {
    sx: function sx(theme) {
      return _extends({
        backgroundColor: lighten(theme.palette.background["default"], 0.06),
        transition: 'all 0.2s ease-in-out',
        '&:hover td': {
          backgroundColor: (tableRowProps == null ? void 0 : tableRowProps.hover) !== false ? theme.palette.mode === 'dark' ? "" + lighten(theme.palette.background["default"], 0.12) : "" + darken(theme.palette.background["default"], 0.05) : undefined
        }
      }, tableRowProps == null ? void 0 : tableRowProps.sx);
    }
  }), row.getVisibleCells().map(function (cell) {
    return React.createElement(MRT_TableBodyCell, {
      cell: cell,
      key: cell.id,
      enableHover: (tableRowProps == null ? void 0 : tableRowProps.hover) !== false,
      tableInstance: tableInstance
    });
  })), renderDetailPanel && !row.getIsGrouped() && React.createElement(MRT_TableDetailPanel, {
    row: row,
    tableInstance: tableInstance
  }));
};

var MRT_TableBody = function MRT_TableBody(_ref) {
  var tableInstance = _ref.tableInstance;
  var getPaginationRowModel = tableInstance.getPaginationRowModel,
      getPrePaginationRowModel = tableInstance.getPrePaginationRowModel,
      _tableInstance$option = tableInstance.options,
      enablePagination = _tableInstance$option.enablePagination,
      muiTableBodyProps = _tableInstance$option.muiTableBodyProps;
  var rows = enablePagination ? getPaginationRowModel().rows : getPrePaginationRowModel().rows;
  var tableBodyProps = muiTableBodyProps instanceof Function ? muiTableBodyProps({
    tableInstance: tableInstance
  }) : muiTableBodyProps;
  return React.createElement(TableBody, Object.assign({}, tableBodyProps), rows.map(function (row) {
    return React.createElement(MRT_TableBodyRow, {
      key: row.id,
      row: row,
      tableInstance: tableInstance
    });
  }));
};

var MRT_TableFooterCell = function MRT_TableFooterCell(_ref) {
  var _ref2, _ref3, _column$columnDef$Foo;

  var footer = _ref.footer,
      tableInstance = _ref.tableInstance;
  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      muiTableFooterCellProps = _tableInstance$option.muiTableFooterCellProps,
      enableColumnResizing = _tableInstance$option.enableColumnResizing;

  var _getState = getState(),
      isDensePadding = _getState.isDensePadding;

  var column = footer.column;
  var mTableFooterCellProps = muiTableFooterCellProps instanceof Function ? muiTableFooterCellProps({
    column: column,
    tableInstance: tableInstance
  }) : muiTableFooterCellProps;
  var mcTableFooterCellProps = column.muiTableFooterCellProps instanceof Function ? column.muiTableFooterCellProps({
    column: column,
    tableInstance: tableInstance
  }) : column.muiTableFooterCellProps;

  var tableCellProps = _extends({}, mTableFooterCellProps, mcTableFooterCellProps);

  return React.createElement(TableCell, Object.assign({
    align: column.columnDefType === 'group' ? 'center' : 'left',
    colSpan: footer.colSpan,
    variant: "head"
  }, tableCellProps, {
    sx: function sx(theme) {
      return _extends({
        backgroundColor: theme.palette.background["default"],
        backgroundImage: "linear-gradient(" + alpha(theme.palette.common.white, 0.05) + "," + alpha(theme.palette.common.white, 0.05) + ")",
        fontWeight: 'bold',
        maxWidth: column.getSize() + "px",
        minWidth: column.getSize() + "px",
        p: isDensePadding ? '0.5rem' : '1rem',
        transition: "all " + (enableColumnResizing ? '10ms' : '0.2s') + " ease-in-out",
        width: column.getSize(),
        verticalAlign: 'text-top'
      }, tableCellProps == null ? void 0 : tableCellProps.sx);
    }
  }), React.createElement(React.Fragment, null, footer.isPlaceholder ? null : (_ref2 = (_ref3 = (_column$columnDef$Foo = column.columnDef.Footer == null ? void 0 : column.columnDef.Footer({
    footer: footer,
    tableInstance: tableInstance
  })) != null ? _column$columnDef$Foo : column.columnDef.footer) != null ? _ref3 : footer.renderFooter()) != null ? _ref2 : null));
};

var MRT_TableFooterRow = function MRT_TableFooterRow(_ref) {
  var _footerGroup$headers;

  var footerGroup = _ref.footerGroup,
      tableInstance = _ref.tableInstance;
  var muiTableFooterRowProps = tableInstance.options.muiTableFooterRowProps; // if no content in row, skip row

  if (!((_footerGroup$headers = footerGroup.headers) != null && _footerGroup$headers.some(function (h) {
    return typeof h.column.columnDef.footer === 'string' && !!h.column.columnDef.footer || h.column.columnDef.Footer;
  }))) return null;
  var tableRowProps = muiTableFooterRowProps instanceof Function ? muiTableFooterRowProps({
    footerGroup: footerGroup,
    tableInstance: tableInstance
  }) : muiTableFooterRowProps;
  return React.createElement(TableRow, Object.assign({}, tableRowProps), footerGroup.headers.map(function (footer) {
    return React.createElement(MRT_TableFooterCell, {
      footer: footer,
      key: footer.id,
      tableInstance: tableInstance
    });
  }));
};

var MRT_TableFooter = function MRT_TableFooter(_ref) {
  var tableInstance = _ref.tableInstance;
  var getFooterGroups = tableInstance.getFooterGroups,
      muiTableFooterProps = tableInstance.options.muiTableFooterProps;
  var tableFooterProps = muiTableFooterProps instanceof Function ? muiTableFooterProps({
    tableInstance: tableInstance
  }) : muiTableFooterProps;
  return React.createElement(TableFooter, Object.assign({}, tableFooterProps), getFooterGroups().map(function (footerGroup) {
    return React.createElement(MRT_TableFooterRow, {
      footerGroup: footerGroup,
      key: footerGroup.id,
      tableInstance: tableInstance
    });
  }));
};

var MRT_Table = function MRT_Table(_ref) {
  var tableInstance = _ref.tableInstance;
  var _tableInstance$option = tableInstance.options,
      enableStickyHeader = _tableInstance$option.enableStickyHeader,
      enableTableFooter = _tableInstance$option.enableTableFooter,
      enableTableHead = _tableInstance$option.enableTableHead,
      muiTableProps = _tableInstance$option.muiTableProps;
  var tableProps = muiTableProps instanceof Function ? muiTableProps({
    tableInstance: tableInstance
  }) : muiTableProps;
  return React.createElement(Table, Object.assign({
    stickyHeader: enableStickyHeader
  }, tableProps), enableTableHead && React.createElement(MRT_TableHead, {
    tableInstance: tableInstance
  }), React.createElement(MRT_TableBody, {
    tableInstance: tableInstance
  }), enableTableFooter && React.createElement(MRT_TableFooter, {
    tableInstance: tableInstance
  }));
};

var useIsomorphicLayoutEffect = typeof window !== 'undefined' ? useLayoutEffect : useEffect;
var MRT_TableContainer = function MRT_TableContainer(_ref) {
  var tableInstance = _ref.tableInstance;
  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      enableStickyHeader = _tableInstance$option.enableStickyHeader,
      idPrefix = _tableInstance$option.idPrefix,
      muiTableContainerProps = _tableInstance$option.muiTableContainerProps;

  var _getState = getState(),
      isFullScreen = _getState.isFullScreen;

  var _useState = useState(0),
      totalToolbarHeight = _useState[0],
      setTotalToolbarHeight = _useState[1];

  var tableContainerProps = muiTableContainerProps instanceof Function ? muiTableContainerProps({
    tableInstance: tableInstance
  }) : muiTableContainerProps;
  useIsomorphicLayoutEffect(function () {
    var _document$getElementB, _document, _document$getElementB2, _document$getElementB3, _document2, _document2$getElement;

    var topToolbarHeight = typeof document !== 'undefined' ? (_document$getElementB = (_document = document) == null ? void 0 : (_document$getElementB2 = _document.getElementById("mrt-" + idPrefix + "-toolbar-top")) == null ? void 0 : _document$getElementB2.offsetHeight) != null ? _document$getElementB : 0 : 0;
    var bottomToolbarHeight = typeof document !== 'undefined' ? (_document$getElementB3 = (_document2 = document) == null ? void 0 : (_document2$getElement = _document2.getElementById("mrt-" + idPrefix + "-toolbar-bottom")) == null ? void 0 : _document2$getElement.offsetHeight) != null ? _document$getElementB3 : 0 : 0;
    setTotalToolbarHeight(topToolbarHeight + bottomToolbarHeight);
  });
  return React.createElement(TableContainer, Object.assign({}, tableContainerProps, {
    sx: _extends({
      maxWidth: '100%',
      maxHeight: enableStickyHeader ? "clamp(350px, calc(100vh - " + totalToolbarHeight + "px), 2000px)" : undefined,
      overflow: 'auto'
    }, tableContainerProps == null ? void 0 : tableContainerProps.sx),
    style: {
      maxHeight: isFullScreen ? "calc(100vh - " + totalToolbarHeight + "px)" : undefined
    }
  }), React.createElement(MRT_Table, {
    tableInstance: tableInstance
  }));
};

var MRT_TablePaper = function MRT_TablePaper(_ref) {
  var tableInstance = _ref.tableInstance;
  var getState = tableInstance.getState,
      _tableInstance$option = tableInstance.options,
      enableToolbarBottom = _tableInstance$option.enableToolbarBottom,
      enableToolbarTop = _tableInstance$option.enableToolbarTop,
      muiTablePaperProps = _tableInstance$option.muiTablePaperProps;

  var _getState = getState(),
      isFullScreen = _getState.isFullScreen;

  useEffect(function () {
    if (typeof window !== 'undefined') {
      if (isFullScreen) {
        document.body.style.overflow = 'hidden';
        document.body.style.height = '100vh';
      } else {
        document.body.style.overflow = 'auto';
        document.body.style.height = 'auto';
      }
    }
  }, [isFullScreen]);
  var tablePaperProps = muiTablePaperProps instanceof Function ? muiTablePaperProps({
    tableInstance: tableInstance
  }) : muiTablePaperProps;
  return React.createElement(Paper, Object.assign({
    elevation: 2
  }, tablePaperProps, {
    sx: _extends({
      transition: 'all 0.2s ease-in-out'
    }, tablePaperProps == null ? void 0 : tablePaperProps.sx),
    style: {
      height: isFullScreen ? '100vh' : undefined,
      margin: isFullScreen ? '0' : undefined,
      maxHeight: isFullScreen ? '100vh' : undefined,
      maxWidth: isFullScreen ? '100vw' : undefined,
      padding: isFullScreen ? '0' : undefined,
      width: isFullScreen ? '100vw' : undefined
    }
  }), enableToolbarTop && React.createElement(MRT_ToolbarTop, {
    tableInstance: tableInstance
  }), React.createElement(MRT_TableContainer, {
    tableInstance: tableInstance
  }), enableToolbarBottom && React.createElement(MRT_ToolbarBottom, {
    tableInstance: tableInstance
  }));
};

var getAllLeafColumnDefs = function getAllLeafColumnDefs(columns) {
  var lowestLevelColumns = columns;
  var currentCols = columns;

  while (!!((_currentCols = currentCols) != null && _currentCols.length) && currentCols.some(function (col) {
    return col.columns;
  })) {
    var _currentCols;

    var nextCols = currentCols.filter(function (col) {
      return !!col.columns;
    }).map(function (col) {
      return col.columns;
    }).flat();

    if (nextCols.every(function (col) {
      return !(col != null && col.columns);
    })) {
      lowestLevelColumns = [].concat(lowestLevelColumns, nextCols);
    }

    currentCols = nextCols;
  }

  return lowestLevelColumns.filter(function (col) {
    return !col.columns;
  });
};
var createGroup = function createGroup(table, column, currentFilterFns) {
  var _column$columns;

  return table.createGroup(_extends({}, column, {
    columns: column == null ? void 0 : (_column$columns = column.columns) == null ? void 0 : _column$columns.map == null ? void 0 : _column$columns.map(function (col) {
      return col.columns ? createGroup(table, col, currentFilterFns) : createDataColumn(table, col, currentFilterFns);
    })
  }));
};
var createDataColumn = function createDataColumn(table, column, currentFilterFns) {
  return (// @ts-ignore
    table.createDataColumn(column.id, _extends({
      filterFn: currentFilterFns[column.id] instanceof Function ? currentFilterFns[column.id] : defaultFilterFNs[currentFilterFns[column.id]]
    }, column))
  );
};
var createDisplayColumn = function createDisplayColumn(table, column) {
  return table.createDisplayColumn(column);
};

var MRT_TableRoot = function MRT_TableRoot(props) {
  var _initialState$current, _initialState$current2, _initialState$isDense, _initialState$isFullS, _initialState$showFil, _initialState$showGlo, _initialState$paginat, _initialState$paginat2, _initialState$paginat3, _initialState$paginat4, _initialState$paginat5, _props$globalFilterFn, _props$state3, _props$state4;

  var _useState = useState(props.idPrefix),
      idPrefix = _useState[0],
      setIdPrefix = _useState[1];

  useEffect(function () {
    var _props$idPrefix;

    return setIdPrefix((_props$idPrefix = props.idPrefix) != null ? _props$idPrefix : Math.random().toString(36).substring(2, 9));
  }, [props.idPrefix]);
  var initialState = useMemo(function () {
    if (!props.enablePersistentState || !props.idPrefix) {
      return props.initialState;
    }

    if (typeof window === 'undefined') {
      if (process.env.NODE_ENV !== 'production') {
        console.error('The MRT Persistent Table State feature is not supported if using SSR, but you can wrap your <MaterialReactTable /> in a MUI <NoSsr> tags to let it work');
      }

      return props.initialState;
    }

    var storedState = props.persistentStateMode === 'localStorage' ? localStorage.getItem("mrt-" + idPrefix + "-table-state") : props.persistentStateMode === 'sessionStorage' ? sessionStorage.getItem("mrt-" + idPrefix + "-table-state") : '{}';

    if (storedState) {
      var parsedState = JSON.parse(storedState);

      if (parsedState) {
        return _extends({}, props.initialState, parsedState);
      }
    }

    return props.initialState;
  }, []);

  var _useState2 = useState((_initialState$current = initialState == null ? void 0 : initialState.currentEditingCell) != null ? _initialState$current : null),
      currentEditingCell = _useState2[0],
      setCurrentEditingCell = _useState2[1];

  var _useState3 = useState((_initialState$current2 = initialState == null ? void 0 : initialState.currentEditingRow) != null ? _initialState$current2 : null),
      currentEditingRow = _useState3[0],
      setCurrentEditingRow = _useState3[1];

  var _useState4 = useState((_initialState$isDense = initialState == null ? void 0 : initialState.isDensePadding) != null ? _initialState$isDense : false),
      isDensePadding = _useState4[0],
      setIsDensePadding = _useState4[1];

  var _useState5 = useState((_initialState$isFullS = initialState == null ? void 0 : initialState.isFullScreen) != null ? _initialState$isFullS : false),
      isFullScreen = _useState5[0],
      setIsFullScreen = _useState5[1];

  var _useState6 = useState((_initialState$showFil = initialState == null ? void 0 : initialState.showFilters) != null ? _initialState$showFil : false),
      showFilters = _useState6[0],
      setShowFilters = _useState6[1];

  var _useState7 = useState((_initialState$showGlo = initialState == null ? void 0 : initialState.showGlobalFilter) != null ? _initialState$showGlo : false),
      showGlobalFilter = _useState7[0],
      setShowGlobalFilter = _useState7[1];

  var _useState8 = useState({
    pageIndex: (_initialState$paginat = initialState == null ? void 0 : (_initialState$paginat2 = initialState.pagination) == null ? void 0 : _initialState$paginat2.pageIndex) != null ? _initialState$paginat : 0,
    pageSize: (_initialState$paginat3 = initialState == null ? void 0 : (_initialState$paginat4 = initialState.pagination) == null ? void 0 : _initialState$paginat4.pageSize) != null ? _initialState$paginat3 : 10,
    pageCount: initialState == null ? void 0 : (_initialState$paginat5 = initialState.pagination) == null ? void 0 : _initialState$paginat5.pageCount
  }),
      pagination = _useState8[0],
      setPagination = _useState8[1];

  var _useState9 = useState(function () {
    return Object.assign.apply(Object, [{}].concat(getAllLeafColumnDefs(props.columns).map(function (c) {
      var _ref, _c$filterFn, _initialState$current3, _c$filterSelectOption, _ref2;

      return _ref2 = {}, _ref2[c.id] = (_ref = (_c$filterFn = c.filterFn) != null ? _c$filterFn : initialState == null ? void 0 : (_initialState$current3 = initialState.currentFilterFns) == null ? void 0 : _initialState$current3[c.id]) != null ? _ref : !!((_c$filterSelectOption = c.filterSelectOptions) != null && _c$filterSelectOption.length) ? MRT_FILTER_OPTION.EQUALS : MRT_FILTER_OPTION.FUZZY, _ref2;
    })));
  }),
      currentFilterFns = _useState9[0],
      setCurrentFilterFns = _useState9[1];

  var _useState10 = useState((_props$globalFilterFn = props.globalFilterFn) != null ? _props$globalFilterFn : MRT_FILTER_OPTION.FUZZY),
      currentGlobalFilterFn = _useState10[0],
      setCurrentGlobalFilterFn = _useState10[1];

  var table = useMemo(function () {
    return createTable();
  }, []);
  var displayColumns = useMemo(function () {
    var _props$localization, _props$localization2, _props$localization3, _props$localization5;

    return [(props.enableRowActions || props.enableEditing && props.editingMode === 'row') && createDisplayColumn(table, {
      Cell: function Cell(_ref3) {
        var cell = _ref3.cell;
        return React.createElement(MRT_ToggleRowActionMenuButton, {
          row: cell.row,
          tableInstance: tableInstance
        });
      },
      header: (_props$localization = props.localization) == null ? void 0 : _props$localization.actions,
      id: 'mrt-row-actions',
      size: 60
    }), (props.enableExpanding || props.enableGrouping) && createDisplayColumn(table, {
      Cell: function Cell(_ref4) {
        var cell = _ref4.cell;
        return React.createElement(MRT_ExpandButton, {
          row: cell.row,
          tableInstance: tableInstance
        });
      },
      Header: function Header() {
        return props.enableExpandAll ? React.createElement(MRT_ExpandAllButton, {
          tableInstance: tableInstance
        }) : null;
      },
      header: (_props$localization2 = props.localization) == null ? void 0 : _props$localization2.expand,
      id: 'mrt-expand',
      size: 50
    }), props.enableRowSelection && createDisplayColumn(table, {
      Cell: function Cell(_ref5) {
        var cell = _ref5.cell;
        return React.createElement(MRT_SelectCheckbox, {
          row: cell.row,
          tableInstance: tableInstance
        });
      },
      Header: function Header() {
        return props.enableSelectAll ? React.createElement(MRT_SelectCheckbox, {
          selectAll: true,
          tableInstance: tableInstance
        }) : null;
      },
      header: (_props$localization3 = props.localization) == null ? void 0 : _props$localization3.select,
      id: 'mrt-select',
      size: 50
    }), props.enableRowNumbers && createDisplayColumn(table, {
      Cell: function Cell(_ref6) {
        var cell = _ref6.cell;
        return cell.row.index + 1;
      },
      Header: function Header() {
        var _props$localization4;

        return (_props$localization4 = props.localization) == null ? void 0 : _props$localization4.rowNumber;
      },
      header: (_props$localization5 = props.localization) == null ? void 0 : _props$localization5.rowNumbers,
      id: 'mrt-row-numbers',
      size: 50
    })].filter(Boolean);
  }, [props.editingMode, props.enableEditing, props.enableExpandAll, props.enableExpanding, props.enableGrouping, props.enableRowActions, props.enableRowNumbers, props.enableRowSelection, props.enableSelectAll, props.localization, table]);
  var columns = useMemo(function () {
    return [].concat(displayColumns, props.columns.map(function (column) {
      return column.columns ? createGroup(table, column, currentFilterFns) : createDataColumn(table, column, currentFilterFns);
    }));
  }, [table, props.columns, currentFilterFns]);
  var data = useMemo(function () {
    var _props$state, _props$state2;

    return ((_props$state = props.state) != null && _props$state.isLoading || (_props$state2 = props.state) != null && _props$state2.showSkeletons) && !props.data.length ? [].concat(Array(10).fill(null)).map(function () {
      return Object.assign.apply(Object, [{}].concat(getAllLeafColumnDefs(props.columns).map(function (c) {
        var _ref7;

        return _ref7 = {}, _ref7[c.id] = null, _ref7;
      })));
    }) : props.data;
  }, [props.data, (_props$state3 = props.state) == null ? void 0 : _props$state3.isLoading, (_props$state4 = props.state) == null ? void 0 : _props$state4.showSkeletons]); //@ts-ignore

  var tableInstance = _extends({}, useTableInstance(table, _extends({
    filterFns: defaultFilterFNs,
    getCoreRowModel: getCoreRowModel(),
    getExpandedRowModel: getExpandedRowModel(),
    getFacetedRowModel: getFacetedRowModel(),
    getFilteredRowModel: getFilteredRowModel(),
    getGroupedRowModel: getGroupedRowModel(),
    getPaginationRowModel: getPaginationRowModel(),
    getSortedRowModel: getSortedRowModel(),
    getSubRows: function getSubRows(row) {
      return row == null ? void 0 : row.subRows;
    },
    //@ts-ignore
    globalFilterFn: currentGlobalFilterFn,
    onPaginationChange: function onPaginationChange(updater) {
      return setPagination(function (old) {
        return functionalUpdate(updater, old);
      });
    }
  }, props, {
    //@ts-ignore
    columns: columns,
    data: data,
    idPrefix: idPrefix,
    initialState: initialState,
    state: _extends({
      currentEditingCell: currentEditingCell,
      currentEditingRow: currentEditingRow,
      currentFilterFns: currentFilterFns,
      currentGlobalFilterFn: currentGlobalFilterFn,
      isDensePadding: isDensePadding,
      isFullScreen: isFullScreen,
      //@ts-ignore
      pagination: pagination,
      showFilters: showFilters,
      showGlobalFilter: showGlobalFilter
    }, props.state)
  })), {
    setCurrentEditingCell: setCurrentEditingCell,
    setCurrentEditingRow: setCurrentEditingRow,
    setCurrentFilterFns: setCurrentFilterFns,
    setCurrentGlobalFilterFn: setCurrentGlobalFilterFn,
    setIsDensePadding: setIsDensePadding,
    setIsFullScreen: setIsFullScreen,
    setShowFilters: setShowFilters,
    setShowGlobalFilter: setShowGlobalFilter
  });

  useEffect(function () {
    if (typeof window === 'undefined' || !props.enablePersistentState) {
      return;
    }

    if (!props.idPrefix && process.env.NODE_ENV !== 'production') {
      console.warn('a unique idPrefix prop is required for persistent table state to work');
      return;
    }

    var itemArgs = ["mrt-" + idPrefix + "-table-state", JSON.stringify(tableInstance.getState())];

    if (props.persistentStateMode === 'localStorage') {
      var _localStorage;

      (_localStorage = localStorage).setItem.apply(_localStorage, itemArgs);
    } else if (props.persistentStateMode === 'sessionStorage') {
      var _sessionStorage;

      (_sessionStorage = sessionStorage).setItem.apply(_sessionStorage, itemArgs);
    }
  }, [props.enablePersistentState, props.idPrefix, props.persistentStateMode, tableInstance]);
  return React.createElement(React.Fragment, null, React.createElement(Dialog, {
    PaperComponent: Box,
    TransitionComponent: Grow,
    disablePortal: true,
    fullScreen: true,
    keepMounted: false,
    onClose: function onClose() {
      return setIsFullScreen(false);
    },
    open: isFullScreen,
    transitionDuration: 400
  }, React.createElement(MRT_TablePaper, {
    tableInstance: tableInstance
  })), !isFullScreen && React.createElement(MRT_TablePaper, {
    tableInstance: tableInstance
  }));
};

var _excluded$5 = ["autoResetExpanded", "columnResizeMode", "defaultColumn", "editingMode", "enableColumnActions", "enableColumnFilters", "enableColumnResizing", "enableDensePaddingToggle", "enableExpandAll", "enableFilters", "enableFullScreenToggle", "enableGlobalFilter", "enableHiding", "enableMultiRowSelection", "enablePagination", "enablePinning", "enableSelectAll", "enableSorting", "enableStickyHeader", "enableTableFooter", "enableTableHead", "enableToolbarBottom", "enableToolbarInternalActions", "enableToolbarTop", "icons", "localization", "persistentStateMode", "positionActionsColumn", "positionPagination", "positionToolbarActions", "positionToolbarAlertBanner"];
var MaterialReactTable = (function (_ref) {
  var _ref$autoResetExpande = _ref.autoResetExpanded,
      autoResetExpanded = _ref$autoResetExpande === void 0 ? false : _ref$autoResetExpande,
      _ref$columnResizeMode = _ref.columnResizeMode,
      columnResizeMode = _ref$columnResizeMode === void 0 ? 'onEnd' : _ref$columnResizeMode,
      _ref$defaultColumn = _ref.defaultColumn,
      defaultColumn = _ref$defaultColumn === void 0 ? {
    minSize: 50,
    maxSize: 1000,
    size: 150
  } : _ref$defaultColumn,
      _ref$editingMode = _ref.editingMode,
      editingMode = _ref$editingMode === void 0 ? 'row' : _ref$editingMode,
      _ref$enableColumnActi = _ref.enableColumnActions,
      enableColumnActions = _ref$enableColumnActi === void 0 ? true : _ref$enableColumnActi,
      _ref$enableColumnFilt = _ref.enableColumnFilters,
      enableColumnFilters = _ref$enableColumnFilt === void 0 ? true : _ref$enableColumnFilt,
      _ref$enableColumnResi = _ref.enableColumnResizing,
      enableColumnResizing = _ref$enableColumnResi === void 0 ? false : _ref$enableColumnResi,
      _ref$enableDensePaddi = _ref.enableDensePaddingToggle,
      enableDensePaddingToggle = _ref$enableDensePaddi === void 0 ? true : _ref$enableDensePaddi,
      _ref$enableExpandAll = _ref.enableExpandAll,
      enableExpandAll = _ref$enableExpandAll === void 0 ? true : _ref$enableExpandAll,
      _ref$enableFilters = _ref.enableFilters,
      enableFilters = _ref$enableFilters === void 0 ? true : _ref$enableFilters,
      _ref$enableFullScreen = _ref.enableFullScreenToggle,
      enableFullScreenToggle = _ref$enableFullScreen === void 0 ? true : _ref$enableFullScreen,
      _ref$enableGlobalFilt = _ref.enableGlobalFilter,
      enableGlobalFilter = _ref$enableGlobalFilt === void 0 ? true : _ref$enableGlobalFilt,
      _ref$enableHiding = _ref.enableHiding,
      enableHiding = _ref$enableHiding === void 0 ? true : _ref$enableHiding,
      _ref$enableMultiRowSe = _ref.enableMultiRowSelection,
      enableMultiRowSelection = _ref$enableMultiRowSe === void 0 ? true : _ref$enableMultiRowSe,
      _ref$enablePagination = _ref.enablePagination,
      enablePagination = _ref$enablePagination === void 0 ? true : _ref$enablePagination,
      _ref$enablePinning = _ref.enablePinning,
      enablePinning = _ref$enablePinning === void 0 ? false : _ref$enablePinning,
      _ref$enableSelectAll = _ref.enableSelectAll,
      enableSelectAll = _ref$enableSelectAll === void 0 ? true : _ref$enableSelectAll,
      _ref$enableSorting = _ref.enableSorting,
      enableSorting = _ref$enableSorting === void 0 ? true : _ref$enableSorting,
      _ref$enableStickyHead = _ref.enableStickyHeader,
      enableStickyHeader = _ref$enableStickyHead === void 0 ? true : _ref$enableStickyHead,
      _ref$enableTableFoote = _ref.enableTableFooter,
      enableTableFooter = _ref$enableTableFoote === void 0 ? true : _ref$enableTableFoote,
      _ref$enableTableHead = _ref.enableTableHead,
      enableTableHead = _ref$enableTableHead === void 0 ? true : _ref$enableTableHead,
      _ref$enableToolbarBot = _ref.enableToolbarBottom,
      enableToolbarBottom = _ref$enableToolbarBot === void 0 ? true : _ref$enableToolbarBot,
      _ref$enableToolbarInt = _ref.enableToolbarInternalActions,
      enableToolbarInternalActions = _ref$enableToolbarInt === void 0 ? true : _ref$enableToolbarInt,
      _ref$enableToolbarTop = _ref.enableToolbarTop,
      enableToolbarTop = _ref$enableToolbarTop === void 0 ? true : _ref$enableToolbarTop,
      icons = _ref.icons,
      localization = _ref.localization,
      _ref$persistentStateM = _ref.persistentStateMode,
      persistentStateMode = _ref$persistentStateM === void 0 ? 'sessionStorage' : _ref$persistentStateM,
      _ref$positionActionsC = _ref.positionActionsColumn,
      positionActionsColumn = _ref$positionActionsC === void 0 ? 'first' : _ref$positionActionsC,
      _ref$positionPaginati = _ref.positionPagination,
      positionPagination = _ref$positionPaginati === void 0 ? 'bottom' : _ref$positionPaginati,
      _ref$positionToolbarA = _ref.positionToolbarActions,
      positionToolbarActions = _ref$positionToolbarA === void 0 ? 'top' : _ref$positionToolbarA,
      _ref$positionToolbarA2 = _ref.positionToolbarAlertBanner,
      positionToolbarAlertBanner = _ref$positionToolbarA2 === void 0 ? 'top' : _ref$positionToolbarA2,
      rest = _objectWithoutPropertiesLoose(_ref, _excluded$5);

  return React.createElement(MRT_TableRoot, Object.assign({
    autoResetExpanded: autoResetExpanded,
    columnResizeMode: columnResizeMode,
    defaultColumn: defaultColumn,
    editingMode: editingMode,
    enableColumnActions: enableColumnActions,
    enableColumnFilters: enableColumnFilters,
    enableColumnResizing: enableColumnResizing,
    enableDensePaddingToggle: enableDensePaddingToggle,
    enableExpandAll: enableExpandAll,
    enableFilters: enableFilters,
    enableFullScreenToggle: enableFullScreenToggle,
    enableGlobalFilter: enableGlobalFilter,
    enableHiding: enableHiding,
    enableMultiRowSelection: enableMultiRowSelection,
    enablePagination: enablePagination,
    enablePinning: enablePinning,
    enableSelectAll: enableSelectAll,
    enableSorting: enableSorting,
    enableStickyHeader: enableStickyHeader,
    enableTableFooter: enableTableFooter,
    enableTableHead: enableTableHead,
    enableToolbarBottom: enableToolbarBottom,
    enableToolbarInternalActions: enableToolbarInternalActions,
    enableToolbarTop: enableToolbarTop,
    icons: _extends({}, MRT_Default_Icons, icons),
    localization: _extends({}, MRT_DefaultLocalization_EN, localization),
    persistentStateMode: persistentStateMode,
    positionActionsColumn: positionActionsColumn,
    positionPagination: positionPagination,
    positionToolbarActions: positionToolbarActions,
    positionToolbarAlertBanner: positionToolbarAlertBanner
  }, rest));
});

export default MaterialReactTable;
//# sourceMappingURL=material-react-table.esm.js.map
